#cython: embedsignature=True
# cython: profile=False

"""Pykaryote Simulations 
"""
from numpy import sum as npsum
from random import random, choice, seed
import numpy as np
import os
import os.path
import shutil
import pickle
import time

# importing all to prepare for recorded actions
from pykaryote.utils.globals import settings as gv

from pykaryote.sim.environment import Environment
from pykaryote.sim.organism import Organism
from pykaryote.utils.analysis import CreativeAnalyzer

def sort_helper(x):
    """
    A simple function to help sorting for breed
    """
    return x[1]


class Simulation(object):
    """
    The Simulation class organizes and runs the complexity simulations.
    
    *Args*:
        ``name`` (str): The name of this simulation
        
        ``data`` (str): The directory in which to store the log and analyzer data 
            the simulation. It is recommended that you choose a local directory
            e.g.  '/tmp/data' to avoid network overhead.
            
        ``config`` (str): the configuration file for the simulation.
        
        ``clear`` (bool): Default True, delete the contents of this simulation's
        data directory before starting.
    """
    
    step = 0
    """A little attribute"""
    
    def __init__(self, config, name="Test", data='.', clear=True,
                 comparison=False, status_callback=None):
        """
        See Class docs

        Comparison `bool`: True if this simulation is part of a comparison.
                When simulation is true CreativeAnalyzer classes know not to
                calculate min, lower, and upper for metrics.
        """
        gv.init_globals(config)
        # Setup the directories for the simulation in the data directory
        self.comparison = comparison
        if clear:
            shutil.rmtree(os.path.join(data, name), ignore_errors=True)
        os.makedirs(os.path.join(data, name, 'logs'))
        os.makedirs(os.path.join(data, name, 'saves'))
        os.makedirs(os.path.join(data, name, 'analyzer'))
        shutil.copy(config, os.path.join(data, name, 'analyzer', 'used.cfg'))
        #simulation setup
        if gv.randomseed == 0:
            gv.randomseed=np.random.randint(100000)
        seed(gv.randomseed)
        np.random.seed(gv.randomseed)
        self.environment = Environment((gv.columns, gv.rows), 
                                       name=name, data=data)
        self.organisms = []
        for org_num in range(gv.num_organisms):
            organism = Organism(self.environment, id=org_num,
                                genome=gv.initial_genome_length,
                                on_movement=self._record_movement)
            self.organisms.append(organism)
        # time_spent_in_gaussian tracks how many codon reads an organism spends
        # inside a chemical mound/gaussian. It is a two dimensional array.
        # The first dimension corresponds to an organism. The seccond
        # corresponds to a chemical gaussian. The last element in the 2nd
        # dimension corresponds to places where there is no gaussian (empty 
        # space)
        self._time_spent_in_gaussian = np.zeros((gv.num_organisms,
                                       len(self.environment.gaussians) + 1))
        self.step = 0
        self.generation = 0
        self.name = name
        self.data = data
        #simulation can be terminated by gv.termination_size, 
        # set to false after 1 termination
        self.terminatable = True
        self.fit_scale = self.create_fitness_scaler()
        for org in self.organisms:
            org.fit_scale = self.fit_scale
        self.analyzer = CreativeAnalyzer(self)
        self.startTime = time.time()
        self.current_complexity_avg = 0
        self.current_irreducible_complexity_avg = 0
        self.previous_gen_times = [0.0, 0.0]
        self.eta = None
        self.timed_out = False
        if status_callback is None:
            # default status callback function prints status each generation
            self._status_callback = self._print_status
        else:
            self._status_callback = status_callback

    def __str__(self):
        """
        Returns a simple string representation of the simulation, see log for 
        more details.
        """
        return "A Simulation"

    def _record_movement(self, location, step, organism_id):
        """Records an organisms movement data.

        Passed as a callback function to Organism.

        Args:
            location: an (x, y) tuple of the organisms location.
            step: the number of codons the organism has read this generation
            organism_id: the id number of the organism.
        """
        # every gv.save_step generations, save data about how much time was
        # spent in each gaussian
        if self.environment.grid_type == 'flat':
            return
        if (self.generation % gv.save_step == 0) and step > 0:
            prev_loc, prev_step = self._prev_location_data[organism_id]
            chem_gaussians = self.environment.gaussians_at_location(prev_loc)
            duration = step - prev_step
            if chem_gaussians is not None:
                for gauss in chem_gaussians:
                    self._time_spent_in_gaussian[organism_id, gauss] += duration
            else:
                self._time_spent_in_gaussian[organism_id,
                                len(self.environment.gaussians)] += duration
        self._prev_location_data[organism_id] = (location, step)

    def _save_gaussian_location_stats(self):
        """Saves stats on how long organisms spent in each chemical gaussian.

        Movement data is recorded in ``self._record_movement``. This method
        is called to save the data at the end of a generation.
        """
        data = self._time_spent_in_gaussian
        if len(data.shape) == 1:
            data = np.array([data])
        self.analyzer.save_as_data(data,
                    'time_spent_in_gaussian_gen_{}.npy'.format(self.generation),
                    format='binary')
        self._time_spent_in_gaussian.fill(0)

    def details(self):
        """
        Prints the details of the simulation in such a way that a new 
        simulation can be constructed from the data.
        
        """
        s = "#Environment::\n" + str(self.environment) + "\n\n"
        s += "\n#Organisms::"
        for org in self.organisms:
            s += '\n' + str(org)
        return s
    
    def log(self, filename):
        """
        Creates a human-readable log file of the simulation in the specified 
        file.
        
        *Args*:
            filename (str): the file in which to save the log.
        """
        f = open(filename, 'w')
        f.write(self.details())
        f.close()

    def _print_status(self, current_gen, target_gen, eta):
        """Prints status information for a running simulation.

        To be used as the default callback for a running simulation to print
        status each generation.
        """
        if self.verbose:
            print 'generation: %d' % current_gen
            print 'eta: %d min %d sec\n' % (eta / 60, eta % 60)

    def run(self, log=True, verbose=True):
        """
        Runs the simulation until it finishes.
        
        *Args*:
            
            ``log`` (boolean): Log the state of the simulation every log_step
                generations
            
            ``verbose`` (boolean): Print generation numbers
        """
        self.verbose=verbose
        while not self.finished_yet():
            self.run_generation(log=log)
        # close data files
        runtime_in_minutes = (time.time() - self.startTime) / 60.0
        self.analyzer.save_as_data(np.array([runtime_in_minutes]),
                                   'total_runtime')
        self.analyzer.close_all()
    
    def finished_yet(self):
        """Returns True if the simulation should end.

        As specified in the config files, a simulation should end after
        generation_limit generations or once complexity grows past 
        complexity_limit or if the estimated total runtime exceeds 
        runtime_limit. If any of these configuration options are zero, there is
        no limit.
        """
        # limit number of generations to run
        if gv.generation_limit and self.generation >= gv.generation_limit:
            return True
        # limit complexity
        if gv.complexity_limit and (self.current_complexity_avg >= 
                                    gv.complexity_limit):
            if gv.generation_limit != 0:
                self.timed_out = True
            return True
        # calculate estimated remaining time
        past_2_gentime_avg = sum(self.previous_gen_times) / 2.0
        if gv.generation_limit != 0.0:
            self.eta = (past_2_gentime_avg * (gv.generation_limit - 
                        self.generation))
        else:
            self.eta = 0.0
        # limit runtime (gv.runtime_limit is in minutes)
        if gv.runtime_limit != 0.0:
            elapsed_time = time.time() - self.startTime
            if ((self.eta + elapsed_time) / 60.0) > gv.runtime_limit:
                if gv.generation_limit != 0:
                    self.timed_out = True
                return True
        return False
        
    def run_generation(self, log=True):
        gen_start_time = time.time()
        # reset previous location data for organisms
        if log:
            if self.generation % gv.log_step == 0:
                self.log(os.path.join(self.data, self.name, 
                         "logs", "gen-%06d.log" % self.generation))
        if self.generation % gv.save_step == 0:
            self._prev_location_data = [None for _ in self.organisms]
            self.save()
        if self.terminatable and (
                                len(self.environment.complexes.complex_lookup) 
                                > gv.termination_size):
            self.save()
            print "%s Terminated" % self.name
            return False
        #check to see if there are any recorded actions for this generation
        try:
            actions = gv.recorded_actions[self.generation]
            #if you add any methods that are meant to be called as recorded actions
            #i.e. some change in the environment, you must add them to ns.
            namespace = {"gv":gv}
            for act in actions:
                exec(act) in namespace
        except KeyError:
            pass
        self.breeders = self.new_generation()
        while self.step < gv.generation_length:
            self.iterate()
        if (self.generation % gv.save_step == 0 
            and self.environment.grid_type != 'flat'):
            self._save_gaussian_location_stats()
        self.analyzer.update()
        # report status
        self._status_callback(self.generation, gv.generation_limit, self.eta)
        #ready to loop again
        self.step = 0
        self.generation += 1
        if self.generation == 1:
            self.previous_gen_times[0] = time.time() - gen_start_time
            self.previous_gen_times[1] = self.previous_gen_times[0]
        else:
            self.previous_gen_times[0] = self.previous_gen_times[1]
            self.previous_gen_times[1] = time.time() - gen_start_time
            
    def new_generation(self):
        """
        Breeds a new generation from the previous. Overwrites the old 
        generation with the new, and returns a list of organisms that 
        reproduced (with organisms that reproduced more than once appearing 
        more than once).
        """
        if self.generation == 0: # the first generation is pregenerated
            return range(gv.num_organisms) # so we don't need to recreate it here
        fit_list = sorted([[i, (self.organisms[i].fitness())**gv.selection_exponent] \
                           for i in range(gv.num_organisms)], key=sort_helper)
        net = sum([i[1] for i in fit_list])
        if net == 0:
            return np.random.randint(50, size=gv.num_organisms)
        cumul_fit = []
        cumul = 0
        for f in fit_list:
            cumul += f[1] / net #this is the percentage of the (fitness**selection_exponent) that is owned by an organism of equal or lesser fitness than this organism
            cumul_fit.append([f[0], cumul])
        new_organisms = []
        breeders = [] 
        cumul_fit[-1][1] = 1 # prevent any numerical issues
        for i in range(gv.num_organisms):
            index = random()
            j = 0
            while index > cumul_fit[j][1]:
                j += 1
            new_organisms.append(self.organisms[cumul_fit[j][0]].breed())
            breeders.append(cumul_fit[j][0])
        for i in range(gv.num_organisms):
            org = new_organisms[i]
            org.id = i
            if random() < gv.swap_probability:
                org.swap(choice(new_organisms))
            
        self.organisms = new_organisms
        return breeders
        
    def iterate(self):
        """
        Advances the simulation one step with each organism reading one 
        step of their genome.
        """
        #run organisms
        for org in self.organisms:
            org.read_genome()
        #update environment
        if gv.diffusion:
            if self.step % gv.diffusion_step == 0:
                self.environment.diffuse()
        self.step += gv.org_step
        
    def summary(self):
        """
        Prints a summary of the current generation, giving their fitnesses and net fitness
        """
        out = "Generation: %i\nFitnesses:\n" % self.generation
        for org in self.organisms:
            out += "%f\n" % org.fitness()
        out += "Net Fitness:%f" % sum([org.fitness() for org in self.organisms])
        return out
        
    def create_fitness_scaler(self):
        """
        Computes the fitness of an "average" primitive organism which has the minimum genome length.
        """
        average_cell = npsum(npsum(self.environment.grid,0),0)/(self.environment.dim[0]*self.environment.dim[1])
        gather_vector = np.zeros(gv.num_chemicals, float)
        for i in range(gv.num_chemicals):
            if i < gv.chem_list[0]:
                gather_vector[i] = (gv.gather_proportion*gv.generation_length/gv.chem_list[0])
        test_org = Organism(self.environment, chemicals = average_cell*gather_vector)
        test_org_chem_fitness = sum(test_org.chem_fitness(test_org.chemicals))
        if test_org_chem_fitness <= 0:
            raise Exception('test_org.chem_fitness = %s'%test_org_chem_fitness)
        return (gv.fitness_scale+gv.genome_fitness_cost*gv.min_genome_length) / test_org_chem_fitness
        
    def save(self):
        """
        Saves a pickled copy of the simulation. This can be loaded via the 
        simulation.load_sim(file) method
        """
        # external functions can't be pickled. remove callback before pickling
        callback = self._status_callback
        self._status_callback = None
        del self._status_callback
        save_file = open(os.path.join(self.data, self.name, 'saves', 
                         'save_gen_%d.p' % self.generation), 'w')
        pickle.dump(self, save_file, protocol =  2)
        # restore callback
        self._status_callback = callback

def load_sim(sim_file):
    """
    Loads and returns a pickled simulation.
    
    *Args*:
        
        ``sim_file`` (str): the filename of the pickled simulation
    """
    load_file = open(sim_file, 'r')
    sim = pickle.load(load_file)
    # set callback function to default
    sim._status_callback = sim._print_status
    return sim



# cython: embedsignature=True
# cython: profile=False

"""Pykaryote Families
"""
from numpy import zeros
from random import random, uniform
    
from pykaryote.utils.globals import settings as gv

import numpy as np
cimport numpy as np

cdef class Family(object):
    """
    A family contains a set of related complexes.
    
    *Args*:
    
        ``id`` (frozenset): a python set containing the id numbers of the protein 
        families that make up this family
        
        ``environment``: a reference to the simulation Environment.
        
    """
    cdef public str name
    cdef public id, environment
    cdef public int length
    """This family's complex length"""
    
    cdef public int size
    """ The number of members of this family"""
    
    cdef public np.ndarray function_vector 
    """The base function vector of this family"""
    
    cdef public list base
    """ 
    For proteins, the first protein in the family, new additions can't be 
    too far from this base.
    """
    
    def __init__(self, id, environment, base = None):
        """
        Constructs Families.
        """
        self.id = id #a python set containing protein family ids
        self.length = len(id)
        self.function_vector = generate_function_vector(self.length)
        self.environment = environment
        self.name = string_from_set(self.id)
        self.size = 0
        self.base = base
        
cdef class FamilyTracker(object):
    """
     Tracks and manages all existing families
     
     *Args*:
        
        ``environment``: A reference to the simulation Evironment.
    """
    cdef public dict family_lookup, family_bindings
    cdef str family_logfile
    cdef int num_proteins
    cdef environment
    
    def __init__(self, environment):
        """
        Initializes the family tracker
        """
        self.family_lookup = {} #has of id:family
        # family_bindings is a dictionary which stores all bindings
        # between two complexes which have already been checked.
        # It memoizes the binds() method.
        # keys are strings representing [family1.name, family2.name]
        self.family_bindings = {} # has all checked family bindings
        self.environment = environment
        #track number of protein families to assign family id's 
        self.num_proteins = 0
        
    def __iter__(self):
        return iter(self.family_lookup)
    
    cpdef Family binds(self, Family fam1, Family fam2):
        """Checks if two families bind and returns the bound family.

        Return None if the families do not bind.
        If their binding hasn't been checked before, initializes it properly
        
        *Args*:
        
            ``fam1id``, ``fam2id``: two families
        """
        cdef str pair_string = str(sorted([fam1.name, fam2.name]))
        try:
            return self.family_bindings[pair_string]
        except KeyError:
            if gv.complex_complex_binding or fam1.length == 1 or fam2.length == 1:
                if fam1.length + fam2.length <= gv.max_complex_length:
                    if random() < gv.family_binding_chance:
                        if fam1.id.isdisjoint(fam2.id):
                            new_fam = self.add_family( id = fam1.id | fam2.id)
                            self.family_bindings[pair_string] = new_fam
                            return new_fam
            self.family_bindings[pair_string] = None
            return None
    
    def add_family(self, id=None, base=None):
        """
        Returns the family matching the given ID. If no such family exists, 
        creates the family.
        
        *Args*:
        
            ``id`` (frozenset or None): For complexes: a frozenset of proteins 
            making up the family. 
            for proteins: None, an id will be generated for the protein
        """
        if id == None:
            id = frozenset([self.num_proteins])
            self.num_proteins += 1
        try: #this family already exists
            return self.family_lookup[string_from_set(id)]
        except KeyError: # create a new family
            new_fam = Family(id, self.environment, base)
            self.family_lookup[new_fam.name] = new_fam
            return new_fam

def generate_function_vector(length):
    """
    Generates a function vector for a family with 'length' proteins (length = 1 => a family of proteins)
    
    *Args*:
    
        ``length`` (int): The complex length that this function vector corresponds to.
    """
    vec = zeros(gv.num_chemicals, dtype=float)
    for i in range(sum(gv.chem_list[:(length + 1)])): #all chemicals harvestable by a complex of length\
        test = random()
        if test < gv.strong_chance: # functional
            vec[i] = (gv.strong_factor ** length) * (gv.range_factor ** uniform(-1, 1))
        elif test < gv.strong_chance + gv.poison_chance: # poison
            vec[i] = -(gv.poison_factor ** length) * (gv.range_factor ** uniform(-1, 1))
        else: # weak
            vec[i] = uniform(-gv.weak_factor * length, gv.weak_factor * length) 
    return vec
    
def string_from_set(set):
    """
    Converts sets (Familiy ids) into strings for comparison/hashing
    """
    return str(sorted(list(set)))



import unittest
import os
import os.path
import shutil
import glob
import tempfile
import ConfigParser as configparser
import matplotlib
matplotlib.use('Agg') # for graphing without an x server
import matplotlib.pyplot as plt
from pkg_resources import resource_filename
from pykaryote.utils.globals import settings as gv
from pykaryote.utils import analysis
from pykaryote.utils import environment_draw
import pykaryote.sim.environment
import pykaryote.sim.simulation
import pykaryote.utils.comparison as comparison

#TODO: add good test coverage for most of the application
#TODO: add tests for petri
SCRIPT_DIR = os.path.dirname(os.path.realpath(__file__))

class TestGlobals(unittest.TestCase):
    def test_init_globals(self):
        """Tests loading of partial sim.cfg files.
        """
        generation_limit = gv.generation_limit
        self.assertEqual(gv.randomseed, 0)
        sim_config = os.path.join(SCRIPT_DIR, 'test_data',
                                  'testInitGlobals-sim.cfg')
        gv.init_globals(sim_config)
        self.assertEqual(gv.randomseed, 999)
        self.assertEqual(gv.generation_limit, generation_limit)

class TestEnvironment(unittest.TestCase):
    """Tests pykaryote.sim.environment.Environment.

    Tests focus on correct creation of the environment grid.
    """
    def setUp(self):
        data_dir = os.path.join(SCRIPT_DIR, 'test_data')
        gv.init_globals(os.path.join(data_dir, 'testEnvironment-sim.cfg'))
        self.env = pykaryote.sim.environment.Environment((gv.rows,
                                                         gv.columns),
        log=False)

    def test_make_flat_grid(self):
        """Tests Environment._make_flat_grid()
        """
        self.env.grid = self.env._make_flat_grid()
        self.assertTrue((self.env.grid == 1.0).all())

    def test_make_gaussian_grid(self):
        """Tests Environment._make_gaussian_grid()
        """
        self.env.grid = self.env._make_gaussian_grid()
        self.assertEqual(self.env.grid[0,0,0], 1.0)
        self.assertTrue(self.env.grid[9,9,0] < 0.0000001)


class TestReadComparison(unittest.TestCase):
    """Tests the comparison.get_config() method, which reads comparison
    configuration files.
    """
    def setUp(self):
        self._base_dir = os.path.join(SCRIPT_DIR, 'test_data')

    def test_get_config(self):
        """Tests comparison.get_config(), which reads a comparison config file.
        """
        options, values = comparison.get_config(os.path.join(self._base_dir,
                                                'get_config_cmp.cfg'))
        self.assertEqual(options, ('test_option',))
        self.assertEqual(values, [(1,), (2,), (3,)])
        options, values = comparison.get_config(os.path.join(self._base_dir,
                                                'get_config_2_cmp.cfg'))
        self.assertEqual(options, ('t1', 't2'))
        self.assertEqual(values, [(1, 2), (3, 4)])


class TestMakeConfig(unittest.TestCase):
    """Tests the comparison.make_cfg() method, which creates config files
    from comparison configuration files.
    """
    def setUp(self):
        self._base_dir = os.path.join(SCRIPT_DIR, 'test_data')
        self._out_filename = os.path.join(self._base_dir,
                                          'temp_made_config.cfg')

    def test_make_config(self):
        """Tests comparison.make_cfg(), which creates config files from
        comparison config files.
        """
        base_config = os.path.join(self._base_dir, 'make_config.cfg')
        comparison.make_cfg(['point_mutate_chance'], [3.14], base_config,
                               self._out_filename)
        config = configparser.RawConfigParser()
        config.read(self._out_filename)
        self.assertTrue(config.has_option('global', 'generation_limit'))
        self.assertTrue(config.has_option('genome', 'point_mutate_chance'))
        self.assertTrue(not config.has_section('organism'))

    def tearDown(self):
        """Delete temporary file.
        """
        try:
            os.remove(self._out_filename)
        except:
            pass


class TestTimeSpentInGaussian(unittest.TestCase):
    """Tests the saving and graphing of data about how long each organism
    spent in each chemical mound/gaussian.
    """
    def setUp(self):
        self.sim_name = 'TestTimeSpentInGaussian'
        self.data_dir = tempfile.mkdtemp(prefix='pykaryote')
        self.test_data_dir = os.path.join(SCRIPT_DIR, 'test_data')
        sim = pykaryote.sim.simulation.Simulation(os.path.join(
                                                  self.test_data_dir,
                            'TestTimeSpentInGaussian-sim.cfg'),
                             name=self.sim_name,
                             data=self.data_dir)
        sim.run(log=False, verbose=False)
        self._analyzer_dir = os.path.join(self.data_dir, self.sim_name,
                                          'analyzer')
        self._ann = analysis.Analyzer([self._analyzer_dir])

    def tearDown(self):
        shutil.rmtree(self.data_dir, ignore_errors=True)

    def test_time_spent_in_gaussian(self):
        """Ensures that the time_spent_in_gaussian data file is correct.

        Checks array dimensions.
        Checks that array sums to gv.generation_length.
        """
        # TODO: add a unit test to test graphing when there is no data to read
        
        for fname in os.listdir(self._analyzer_dir):
            if fname[:27] == 'time_spent_in_gaussian_gen_':
                data = self._ann.quick_read(fname, ndmin=2)
                self.assertEqual(data.shape,
                                 (gv.num_organisms, len(gv.gaussians) + 1))
                for row in data:
                    self.assertEqual(row.sum(), gv.generation_length)

    def test_plot_time_spent_in_gaussian(self):
        """Test the plotting of ``time_spent_in_gaussian`` data.
        """
        self._ann.plot_time_spent_in_gaussian()
        plt.savefig(os.path.join(self.data_dir, 'time_spent_in_gaussians.png'))

    def test_flat_environment(self):
        """Tests plotting with a flat environment.

        Since there are no gaussians, no data should be recorded, and the graph
        should be blank.
        """
        sim_name = 'TestTimeSpentInFlat'
        analyzer_dir = os.path.join(self.data_dir, sim_name, 'analyzer')
        sim = pykaryote.sim.simulation.Simulation(os.path.join(
                                                  self.test_data_dir,
                                                'TestTimeSpentInFlat-sim.cfg'),
                                            name=sim_name, data=self.data_dir)
        sim.run(log=False, verbose=False)
        time_spent_data = [f for f in os.listdir(analyzer_dir)
                            if f[:27] == 'time_spent_in_gaussian_gen_']
        self.assertEqual(time_spent_data, [])
        ann = analysis.Analyzer([analyzer_dir])
        ann.plot_time_spent_in_gaussian()
        plt.savefig(os.path.join(self.data_dir,
                    'time_spent_in_gaussian_flat.png'))
        shutil.rmtree(os.path.join(self.data_dir, sim_name),
                      ignore_errors=True)

    def tearDown(self):
        """Removes simulation data.
        """
        shutil.rmtree(os.path.join(self.data_dir, self.sim_name),
                      ignore_errors=True)

#TODO: test running and analyzing simulations with only one generation.


class TestEnvironmentDraw(unittest.TestCase):
    """Tests pykaryote.utils.environment_draw.
    """
    def setUp(self):
        self.data_dir = os.path.join(SCRIPT_DIR, 'test_data')
        gv.init_globals(os.path.join(self.data_dir,
                        'testEnvironmentDraw-sim.cfg'))
        self.env = pykaryote.sim.environment.Environment((gv.columns,
                                                         gv.rows), log=False)

    def test_draw_environment(self):
        temp_dir = tempfile.mkdtemp(prefix='pykaryote')
        env = pykaryote.sim.environment.Environment((gv.columns, gv.rows),
                                                    log=False)
        filename = os.path.join(temp_dir, 'environment.png')
        environment_draw.draw_environment(env.grid, out_filename=filename)
        shutil.rmtree(temp_dir, ignore_errors=True)

    def test_make_chemical_colors(self):
        colors = environment_draw.make_chemical_colors(1)
        self.assertEqual(colors, [(191, 64, 64)])
        colors = environment_draw.make_chemical_colors(3)
        expected = [(191, 64, 64), (64, 191, 64), (64, 64, 191)]
        self.assertEqual(colors, expected)


class TestAnalyzerQuickRead(unittest.TestCase):
    """Tests pykaryote.utils.analysis.Analyzer.quick_read()

    Tests are focused on calls where mode='threshold', which is used to
    gather data on how many generations it takes a metric to reach a threshold
    value.

    Test coverage is very minimal.
    """

    def setUp(self):
        base_dir = os.path.join(SCRIPT_DIR, 'test_data', 'analyzer_quick_read')
        gv.init_globals(os.path.join(base_dir, 'sim.cfg'))
        analyzer_dirs = []
        for x in range(12):
            analyzer_dirs.append(os.path.join(base_dir, 'ann{}'.format(x)))
        self.ann = analysis.Analyzer(analyzer_dirs)
        aggregate = self.ann.quick_read('gens_until_complexity_avg',
                                        mode='threshold')
        self.mean = aggregate[0]
        self.stddev = aggregate[1]
        self.blanks = aggregate[2]
        self.complexity = self.ann.quick_read('complexity_avg')

    def test_shape(self):
        """Make sure the resulting data is the right shape.
        """
        self.assertEqual(self.mean.shape, (6,))
        self.assertEqual(self.stddev.shape, (6,))
        self.assertEqual(self.blanks.shape, (6,))
        self.assertEqual(self.complexity.shape, (1000,))
        
    def test_blanks(self):
        """Make sure the right number of blanks is reported.
        """
        self.assertTrue(self.blanks[0] == 0)
        self.assertTrue(self.blanks[1] == 0)
        self.assertTrue(self.blanks[2] == 0)
        self.assertTrue(self.blanks[3] == 0)
        self.assertTrue(self.blanks[4] > 0)
        self.assertTrue(self.blanks[5] == 12)

    def test_complexity(self):
        """Make sure complexity_avg is right.
        """
        self.assertEqual(self.complexity[0], 0.0)
        self.assertTrue(self.complexity[1] != 1.019066099611200043e-03)
        self.assertEqual(self.complexity[999], 3.462880888059239659e+00)

    def test_means(self):
        """Make sure the correct means are reported.
        """
        self.assertTrue(self.mean[5] == 1000.0)

    def test_variable_length_data(self):
        """Test that data with varying length (number of generations) is
        handled correctly.

        For simulations where a time or complexity limit is used instead of
        a generation limit.
        """
        base_dir = os.path.join(SCRIPT_DIR, 'test_data')
        data_folders = glob.glob(os.path.join(base_dir,
                                'analyzer_variable_length', '*', 'analyzer'))
        ann = analysis.Analyzer(data_folders)
        data = ann.quick_read('complexity_avg')
        self.assertEqual(data.shape, (78,))
        data = ann.quick_read('chemical_complexity')

def run():
    """Runs unit tests with a command line interface.
    """
    unittest.main()

if __name__ == '__main__':
    run()



# cython: embedsignature=True
# cython: profile=False

"""Pykaryote Complexes
"""
from random import choice, uniform, random
import os.path
from pykaryote.utils.globals import settings as gv

import numpy as np
cimport numpy as np

nint = np.int
ctypedef np.int_t nint_t

nfloat = np.float 
ctypedef np.float_t nfloat_t

cdef class Complex:
    """
    The Complex class represents combinations of proteins from 1 (a protein) 
    to many.
    
    Each instance of Complex is stored in the ComplexTracker class, and 
    referenced by the various organisms that build it.
    
    *Args*:
    
        ``composition`` (list): The chemical list versions of the proteins 
        making up the complex
        
        ``function`` (array): The complex's function vector
        
        ``family``: The family to which the complex belongs
    """
    
    def __init__(self, composition, function, family):
        """
        Initializes the Complex
        """
        cdef float x
        self.function = function
        self.family = family
        self.length = len(composition)
        # TODO: store as array/tuple instead of string
        self.name = str(composition)
        self.is_functional = 0 
        if self.length != 1:
            self.binding_affinity = uniform(gv.min_binding_affinity, 
                                            gv.max_binding_affinity)
        for x in self.function:
            if x >= gv.weak_factor*self.length*gv.mutate_factor:
                self.is_functional = 1
 
    
    def __str__(self):
        id = str(list(self.family.id))
        fstring = function_string(self.function)
        return "%s\nFunction: %s\nFamily: %s\n\n" % (self.name, fstring, id) 
        

cdef class ComplexTracker:
    """
    Tracks and manages all existing proteins
    
    *Args*:
    
        ``environment``: A reference to the simulation Environment
        
        ``name`` (str): The name of the simulation, used for logging
        
        ``data`` (str): The data directory of the simulation, used for logging
    """
    
    def __init__(self, environment, str name="", str data="", log=True):
        """
        Creates a new Complex tracker for the specified environment.
        """
        self.complex_lookup = {} # str:complex
        if log:
            self.complex_logfile = open(os.path.join(
                                    data, name, 'complexes.log'), 'w')
        else:
            self.complex_logfile = None
        self.environment = environment
        
    def __str__(self):
        return str(self.complex_lookup)

    def __del__(self):
        # TODO: use 'with' to open files instead of explicitly closing
        if self.complex_logfile is not None:
            self.complex_logfile.close()
        
    def get_complex(self, str complex, family = None):
        """Return a complex by name, creating it if it does not exist.

        If looking up a complex of length 1, let family = None.
        
        *Args*:
        
            ``complex`` (str): A STRING representation of the complex's 
            composition
        
            ``family``: None for proteins (len 1 complexes)
            A Family for complexes - the family to which the complex will belong
        """
        try:
            return self.complex_lookup[complex]
        except KeyError:
            if family:
                return self.new_complex(complex, family)
            else:
                return self.new_protein(complex)
             
        
    cdef Complex new_protein(self, str pname):
        """
        Adds a new protein to the known complexes, creating a new family if 
        necessary.
        
        *Args*:
        
            ``pname`` (str): The string version of the protein list
        """
        # TODO: don't use eval
        cdef list plist = eval(pname[1:-1]) #internal list, our current protein.
        siblings = []
        #check to see if the new protein has any relations
        for tp in self.complex_lookup.itervalues():
            if tp.length == 1:
                #tpl is the protein against which we shall check.
                tpl = eval(tp.name)[0]
                #are they siblings?
                if is_relative(tpl, plist, gv.sibling_distance): 
                    #is plist within the bounds of the family?
                    if is_relative(tp.family.base, plist, 
                                   gv.max_relation_distance): 
                        siblings.append(tp)
        #add to existing family
        if len(siblings) > 0 and random() > gv.separation_chance: 
            ancestor = choice(siblings)
            fam = ancestor.family
            function = mutate_function_vector(ancestor.function, 1)
        # new family
        else: 
            fam = self.environment.families.add_family(base = plist)
            function = mutate_family_vector(fam.function_vector)
        new_pro = Complex([plist], function, fam)
        fam.size += 1
        self.complex_lookup[new_pro.name] = new_pro
        self.log(new_pro)
        return new_pro
        
    cdef void log(self, Complex complex):
        """
        Records the complex in the log file
        """
        if self.complex_logfile is not None:
            self.complex_logfile.write(str(complex))
    
    cdef Complex new_complex(self, str complex, family):
        """
        Adds the complex to the list of known complexes, creating a new family 
        if necessary.
        
        *Args*:
        
            ``composition`` (str): The string representation of the complex
        
            ``family``: The family to which the complex will belong
        """
        #Check if this is a new way to make an existing complex
        if complex in self.complex_lookup: 
            return self.complex_lookup[complex]
        function = mutate_family_vector(family.function_vector)
        new_comp = Complex(eval(complex), function, family)
        family.size += 1
        self.complex_lookup[new_comp.name] = new_comp
        self.log(new_comp)
        return new_comp
            
def function_string(function):
    """
    Converts a function vector into a string for displaying
    """
    fstring = "["
    for f in function:
        fstring += "%4.4f, "%f
    return fstring[:-2]+"]"

def mutate_function_vector(base_vec, length):
    """
    Tweaks the supplied vector to create a new function vector.
    Used for individualizing relative functions for a new protein
    
    *Args*:
    
        ``base_vec``: The function vector to be tweaked
        
        ``length`` (int): the length of the complex in question
    """
    new_vec = np.zeros(gv.num_chemicals, dtype=float)
    scap = gv.strong_factor ** length * gv.range_factor
    smin = gv.strong_factor ** length / gv.range_factor
    wcap = gv.weak_factor
    wmin = -gv.weak_factor
    pcap = gv.poison_factor ** length * gv.range_factor
    pmin = gv.poison_factor ** length / gv.range_factor
    for i in range(gv.chem_list[0] + gv.chem_list[1]):
        new_value = base_vec[i] * gv.mutate_factor ** uniform(-1, 1)
        if base_vec[i] >= smin: #if its in the strong range, keep it there.
            if new_value > scap: 
                new_value = scap
            elif new_value < smin:
                new_value = smin
        elif -base_vec[i] >= pmin: #likewise for poison range.
            if -new_value > pcap: 
                new_value = -pcap
            elif -new_value < pmin:
                new_value = -pmin
        else:
            if new_value > wcap: #likewise for weak range.
                new_value = wcap
            elif new_value < wmin:
                new_value = wmin
        new_vec[i] = new_value
    return new_vec
    
cdef is_relative(list p1, list p2, int max_diff):
    """
    Checks to see if the two proteins (represented by their lists) are related
    (fewer than max_diff point differences)
    
    *Args*:
    
        ``p1``, ``p2`` (list): the proteins to be compared
        
        ``max_diff`` (int): The maximum number of differences
    """
    cdef int l, i, diff
    l = len(p1)
    if len(p2) == l:
        diff = 0
        for i in xrange(l):
            if p1[i] != p2[i]:
                diff += 1
        if diff <= max_diff:
            return True
    return False

def mutate_family_vector(vec):
    """
    mutates the family function vector to create a complex vector
    """
    return np.array([v * gv.mutate_factor**uniform(-1,1) for v in vec], float)



#!/usr/bin/env python
"""Code to be run by worker processors.

Worker processors execute jobs and report status back to the master processor.
"""
from mpi4py import MPI
import multiprocessing
import os.path
import os
import sys
import shutil
import time
import tempfile
import tags

import numpy as np
import traceback

# import hack to get matplotlib graphing to work over ssh
import matplotlib as mpl
mpl.use('Agg')

from pykaryote.utils.comparison import Grapher, get_config
from pykaryote.sim.simulation import Simulation
from pykaryote.utils.analysis import aggregate_batch
from pykaryote.utils.comparison import AnalyzerAggregator

comm = MPI.COMM_WORLD


class Worker(object):
    """Runs jobs on worker processors and reports status back to the master.

    Jobs are received from the master thread. Typical jobs include running
    simulations and graphing comparison batches.
    """
    def __init__(self, data_dir, status_interval=10):
        """
        Args:
            status_interval (int): The minimum number of seconds to wait before
            sending status data to the master thread.
        """
        self._status_interval = status_interval
        self._last_status = time.time()
        self._thread = None
        self._job = None
        self._wait_for_orders = True
        self._dest_dir = data_dir

    def run(self):
        """Runs the current job.
        """
        while self._wait_for_orders:
            status = MPI.Status()
            message = comm.recv(source=tags.MASTER_ID, tag=MPI.ANY_TAG,
                                status=status)
            if status.tag == tags.JOB_TAG:
                if self._job is None:
                    self._run_job(message)
                else:
                    # already running a simulation
                    comm.send(message, tags.MASTER_ID, tag=tags.FAILED_TAG)
            elif status.tag == tags.SHUTDOWN_TAG:
                self._wait_for_orders = False

    def _status_update(self, current_gen, target_gen, eta):
        """Receives status from a running simulation.

        This is a callback function to be passed to simulation.run()
        Reports status to master in intervals of at least self._status_interval
        seconds.
        """
        if time.time() - self._last_status > self._status_interval:
            #TODO: use buffered Send/Recv here
            status = np.array([current_gen, target_gen, int(eta)], dtype=int)
            comm.send(status, tags.MASTER_ID,
                       tag=tags.STATUS_TAG)
            self._last_status = time.time()

    def _run_job(self, job):
        """Runs a job.

        Jobs can be either comparison jobs, or graphing jobs. A comparison
        job runs a single simulation as part of a comparison. A graphing job
        graphs a completed comparison.

        Simulations are run in a temporary directory and then copied back
        to the destination directory if they finish successfully.
        """
        self._job = job
        if job.get('type') == 'comparison_graph':
            self._run_comparison_graph_job(job)

        elif job.get('type') == 'comparison':
            self._run_comparison_job(job)
        else:
            self._job = None
            print 'unrecognized job type: {}'.format(job.get('type'))
            comm.send(job, tags.MASTER_ID, tag=tags.FAILED_TAG)

    def _run_comparison_job(self, job):
        """Runs a simulation which is part of a larger comparison.

        If exceptions occur, they are caught and the job is reported as failed.
        Timed out jobs are also treated as failed.
        """
        # create simulation
        dest_data_dir = os.path.join(self._dest_dir, job['comparison_name'], 
                                     'sim-{}'.format(job['sim_num']))
        temp_data_dir = tempfile.mkdtemp(prefix='simulation_data')
        sim_name = 'run-{}'.format(job['run_num'])
        self._sim = Simulation(job['config_filename'], data=temp_data_dir,
                               name=sim_name, clear=True, comparison=True,
                               status_callback=self._status_update)
        sim_dir = os.path.join(temp_data_dir, sim_name)

        # run the simulation, catching and reporting any errors
        try:
            self._sim.run(log=False, verbose=False)
            # timed out simulations are treated as failed.
            if self._sim.timed_out:
                raise RuntimeError('Simulation timed out.')
            # copy simulation data back to shared directory for graphing
            shutil.move(sim_dir, os.path.join(dest_data_dir, sim_name))
            self._sim = None
            self._job = None
            comm.send(job, tags.MASTER_ID, tag=tags.SUCCESS_TAG)
        except Exception as e:
            # print stack trace and report job as failed
            exc_type, exc_value, exc_traceback = sys.exc_info()
            if e.args[0] != 'Simulation timed out.':
                traceback.print_exception(exc_type, exc_value, exc_traceback)
            # write error to a logfile for that simulation
            with open(os.path.join(sim_dir, 'error.log'), 'w') as f:
                traceback.print_exception(exc_type, exc_value, exc_traceback,
                                          file=f)
            self._sim = None
            self._job = None
            # copy failed simulation to the 'failed' directory
            failed_dir = os.path.join(self._dest_dir, job['comparison_name'],
                                      'failed', 'sim-{}'.format(job['sim_num']),
                                      'run-{}'.format(job['run_num']))
            shutil.move(sim_dir, failed_dir)
            comm.send(job, tags.MASTER_ID, tag=tags.FAILED_TAG)
        shutil.rmtree(temp_data_dir)

    def _run_comparison_graph_job(self, job):
        """Wraps a comparison graphing job in a try/catch clause, reporting
        errors if they occur.
        """
        try:
            self.graph_comparison(job['comparison_name'], values=job['values'],
                                  clean=job['clean'])
            self._job = None
            comm.send(job, tags.MASTER_ID, tag=tags.SUCCESS_TAG)
        except Exception as e:
            print e
            traceback.print_exc()
            self._job = None
            comm.send(job, tags.MASTER_ID, tag=tags.FAILED_TAG)

    def graph_comparison(self, cmp_name, values=None, clean=True):
        """Aggregate and graph a completed comparison.
        """
        comparison_dir = os.path.join(self._dest_dir, cmp_name)
        # try not to aggregate if an aggregate directory already exists
        if os.path.isfile(os.path.join(self._dest_dir, cmp_name,
                          'comparative-analyzer')):
            try:
                self._graph_comparison
            except Exception:
                self._aggregate_comparison(comparison_dir, cmp_name, values)
                self._graph_comparison(comparison_dir)
        else:
            self._aggregate_comparison(comparison_dir, cmp_name, values)
            self._graph_comparison(comparison_dir)

        if clean:
            # remove simulation data, leaving the comparison-aggregate dir
            for batch_dir in os.listdir(comparison_dir):
                if batch_dir[:4] == 'sim-':
                    shutil.rmtree(os.path.join(comparison_dir, batch_dir),
                                  ignore_errors=True)

    def _aggregate_comparison(self, comparison_dir, cmp_name, values=None):
        """Aggregate a completed comparison.

        Args:
            values: A list of values used for varied options from the
            comparison.
        """
        sim_config = os.path.join(comparison_dir, 'sim_configs', 'sim-0.cfg')
        # combine each batch of simulations from the comparison by averaging
        # its analyzer directories
        batch_sim_dirs = []
        for batch_sim_dir in os.listdir(comparison_dir):
            if batch_sim_dir[:4] == 'sim-':
                batch_sim_dirs.append(batch_sim_dir)
                # get the analyzer directory for each run from this batch
                analyzer_dirs = []
                for run_dir in os.listdir(os.path.join(comparison_dir, 
                                          batch_sim_dir)):
                    if run_dir[:4] == 'run-':
                        analyzer_dirs.append(os.path.join(comparison_dir, 
                                             batch_sim_dir, run_dir, 
                                             'analyzer'))
                # average into a single analyzer directory
                output_analyzer_dir = os.path.join(comparison_dir,
                                                   batch_sim_dir, 'analyzer')
                aggregate_batch(analyzer_dirs, output_analyzer_dir)

        if values is None:
            cmp_filename = os.path.join(comparison_dir, 'cmp.cfg')
            options, values = get_config(cmp_filename)
        
        # combine into a comparison-aggregate dir
        agg = AnalyzerAggregator(self._dest_dir, cmp_name, values, sim_config)
        agg.compile_all()

    def _graph_comparison(self, comparison_dir):
        sim_config = os.path.join(comparison_dir, 'sim_configs', 'sim-0.cfg')
        g = Grapher(comparison_dir, sim_config, verbose=False)
        g.save_all()



# cython: embedsignature=True
# cython: profile=False

"""Pykaryote Environments
"""
import math
import numpy as np
import itertools
from numpy.random import randint
from random import random

from pykaryote.sim.complexes import ComplexTracker
from pykaryote.sim.families import FamilyTracker
from pykaryote.utils.globals import settings as gv

cimport numpy as np

def distance(p0, p1):
    """Calculate the distance between two points.
    """
    return math.sqrt((p0[0] - p1[0])**2 + (p0[1] - p1[1])**2)

cdef class Environment(object):
    """
    The environment in which organisms live.
    
    *Args*:
    
        ``size`` (tuple): The dimensions of the environment. (X,Y) or (X,Y,Z)
            currently supported only for dim = 2 
    
        ``grid`` (array): The chemical grid of the environment. If None, the 
            grid will be created based on the parameters in the configuration
            file. The last dimension holds the number of each type of
            chemicals.

            For example, to access the number of chemical-2 at location (5, 7),
            use::

                num_chemical_2 = environment.grid[5, 7, 2]

    
    """
    
    cdef public tuple dim
    
    cdef public np.ndarray grid

    cdef public list gaussians

    cdef public str grid_type
    
    cdef public complexes, families

    def __init__(self, size, grid=None, name="",data="", log=True):
        """Creates the environment grid and populates it with chemicals.
        """
        if len(size) != 2:
            raise(Exception("Environments with dimension != 2 are not "
                  " supported"))
        self.dim = size
        self.gaussians = []
        self.grid_type = gv.grid_type
        # create the environment grid
        if grid is None:
            if self.grid_type in ('flat', 'uniform'):
                self.grid = self._make_flat_grid(value=gv.base_concentration)
            elif self.grid_type in ('gauss', 'gaussian'):
                self.gaussians = gv.gaussians
                self.grid = self._make_gaussian_grid()
            else:
                raise ValueError('Unsupported grid type: {}'.format(
                                 self.grid_type))
        else:
            self.grid = np.array(grid, dtype=float)
        self.complexes = ComplexTracker(self, name=name, data=data, log=log)
        self.families = FamilyTracker(self)
                
    def __str__(self):
        """Return an ascii representation of the environment grid.
        """
        return str(self.grid.tolist()) #proteins are saved separately

    def _make_flat_grid(self, value=1.0):
        """Creates an environment grid with uniformly distributed chemicals.
        """
        return value * np.ones(self.dim + (gv.num_chemicals,), dtype=float)

    def _make_gaussian_grid(self):
        """Creates a grid with chemicals distributed along gaussians.

        Information on the gaussians is read from gv.gaussians, which is a
        list of gaussians to create. Each gaussian is a tuple of::

            (chemical, (center_x, center_y), radius)

        """
        def makeGaussian(size, fwhm = 3, center=None, infinate=True):
            """ Make a square gaussian kernel.
         
            size is the length of a side of the square
            fwhm is full-width-half-maximum, which
            can be thought of as an effective radius.

            When ``infinate`` is true, the gaussian extends infinitely past
            the radius.

            From https://gist.github.com/andrewgiessel/4635563
            """
         
            x = np.arange(0, size, 1, float)
            y = x[:,np.newaxis]
            
            if center is None:
                x0 = y0 = size // 2
            else:
                x0 = center[0]
                y0 = center[1]
            
            gauss = np.exp(-4*np.log(2) * ((x-x0)**2 + (y-y0)**2) / fwhm**2)
            if not infinate:
                for point in itertools.product(range(size), range(size)):
                    if distance(point, (x0, y0)) > fwhm:
                        gauss[point[1], point[0]] = 0.0
            return gauss

        grid = np.zeros(self.dim + (gv.num_chemicals,), dtype=float)
        for chemical, center, radius in self.gaussians:
            gauss = makeGaussian(max(self.dim), radius, center=center)
            #Note: gaussians do not wrap around environment edges
            for x, y in itertools.product(xrange(self.dim[0]), xrange(self.dim[1])):
                grid[x, y, chemical] += gauss[x, y]
        return grid
        #TODO: how should the concentration/ammount of chemicals be configured?
        # should it be a percentage out of 1.0, or something else?

    def diffuse(self):
            """Diffuses chemicals between cells.

            Chemicals diffuse between cells at a controlled rate.
            Not fully implemented
            Update to use numpy array addition
            """
            rmax = self.dim[0]
            cmax = self.dim[1]
            tgrid = []
            for r in range(rmax):
                trow = []
                for c in range(cmax):
                    tcell = self.grid[r][c].chemicals*(1-4*gv.diffusion_coefficient)
                    tcell += self.grid[(r-1)%rmax][c].chemicals*gv.diffusion_coefficient #up
                    tcell += self.grid[r][(c+1)%cmax].chemicals*gv.diffusion_coefficient #right
                    tcell += self.grid[(r+1)%rmax][c].chemicals*gv.diffusion_coefficient #down
                    tcell += self.grid[r][(c-1)%cmax].chemicals*gv.diffusion_coefficient #left
                    trow.append(tcell)
                tgrid.append(trow)
            for r in range(rmax):
                for c in range(cmax):
                    self.grid[r][c] = tgrid[r][c]
        
    def random_location(self):
        """Returns a random location in the environment grid.
        """
        return [randint(0, self.dim[0]), randint(0, self.dim[1])]

    def gaussians_at_location(self, location):
        """Returns a lisf of the chemical gaussians at the given cell location.

        If no chemical gaussians are present, returns None.
        """
        gaussians = []
        gaussian_index = 0
        for chemical, center, radius in self.gaussians:
            if distance(location, center) <= radius:
                gaussians.append(gaussian_index)
            gaussian_index += 1
        if gaussians == []:
            return None
        else:
            return gaussians




#cython: embedsignature=True
# cython: profile=False

"""Pykaryote Organisms
"""
from numpy import abs
from random import choice

from pykaryote.sim.complexes import function_string
from pykaryote.sim.genome import mutate_genome, generate_genome, swap_gene

from pykaryote.utils.globals import settings as gv

import numpy as np
cimport numpy as np

from pykaryote.sim.complexes cimport Complex, ComplexTracker

nint = np.int
ctypedef np.int_t nint_t

nfloat = np.float
ctypedef np.float_t nfloat_t

import random

cdef class CEntry:
    cdef public Complex complex
    cdef public float amount
    cdef public list binding_list
    
    def __init__(self, complex, float amount):
        self.complex = complex
        self.amount = amount
        self.binding_list = [] # list of Bonds which this complex can make

cdef class Bond:
    cdef public float affinity
    cdef public CEntry other, result
    
    def __init__(self, affinity, other, result):
        self.affinity = affinity
        self.other = other # the Complex being bound to
        self.result = result # the Complex resulting from this binding

cdef class Organism:
    """
    This class represents an organism in the simulation. It has a location, 
    a genome, etc.
    
    The self.complexes data structure is a bit complicated and deserves 
    explanation.
    
    complexes has an entry for each complex that the organism has. 
    The entries are tuples with the following elements.
    
    #. the complex
    #. the current amount of the complex possessed by the organism
    #. a list of all complexes in the organism that [1] binds to, 
        each element has:
            #. The binding affinity
            #. a reference to the entry for the other complex in self.complexes
            #. a reference to the entry for the result complex in self.complexes

    *Args*:
    
        ``environment``: The environment in which this organism lives, 
            used as a reference
    
        ``row`` (int): The row location in the environment. if None it will be 
            chosen randomly
        
        ``column`` (int): The column location in the environment. if None it 
            will be chosen randomly
    
        ``genome`` (int or list): The genome of the organism, 
        if a list, the genome will stay that way
        if an int, a genome of that length will be generated
        
        ``chemical`` (list): The starting amount of chemicals owned by the organism. 
        If None, the organism owns 0.0 of all chemicals
        
        ``fit_scale`` (float): The fitness scaling factor. See the documentation for 
        further details.
        
        ``id`` (int): The id # for this organism w/in the generation.
        
        ``ancestry`` (list): the list of ancestor id #'s
        
        ``data`` (list): a list of control values for the organism, used only 
        for reconstructing an organism.
    """
    
    cdef public environment
    
    cdef public dict complexes
    
    cdef list current_protein
    cdef public list ancestry
    cdef public int row, column,  move_count, mode, pos, len, id, _codon_read_number
    cdef public int moves, builds, gathers, failed_builds 
    cdef public np.ndarray genome
    cdef public np.ndarray chemicals, cell, function
    cdef public float fit_scale
    cdef object _on_movement_callback

    def __init__(self, environment, row=None, column=None,
        genome=gv.initial_genome_length, chemicals=None, fit_scale=1.0,
        id=-1, ancestry=[], data=None, on_movement=None):
        """
        Creates a new organism, generating as much of the data as necessary.

        Args:
            ``on_movement`` - Callback function used to report an organisms
            location. Called whenever an organism moves.
        
        """
        self.environment = environment
        if row == None:
            self.row = random.randint(0, environment.dim[0] - 1)
        else:
            self.row = row
        if column == None:
            self.column = random.randint(0, environment.dim[1] - 1)
        else:
            self.column = column
        if type(genome) == int:
            self.genome = np.array(generate_genome(genome), dtype=nint) 
        else:
            self.genome = np.array(genome, dtype = nint)
        self.len = len(self.genome)
        if chemicals == None:
            self.chemicals = np.zeros(gv.num_chemicals, dtype=nfloat)
        else:
            self.chemicals = np.array(chemicals, dtype=nfloat)
        self.mode = 0 #handle gv.num_gather=0 correctly
        self.pos = 0
        self.move_count = 0
        self.current_protein = []
        self.complexes = {} #name:[complex, quantity]
        self.function = create_function_vector()
        self.cell = self.environment.grid[self.row, self.column]
        self.moves = 0 #data tracking variable
        self.builds = 0
        self.failed_builds = 0
        self.gathers = 0
        self.fit_scale = fit_scale
        self.id = id
        self.ancestry = ancestry
        self._on_movement_callback = on_movement
        self._codon_read_number = 0
                
    cdef bind_complex(self, CEntry complex, float amount):
        """
        Properly binds 'amount' of 'complex' to other complexes in the cell. 
        The total amount bound is kept under a maximum value and is divided 
        appropriately among possible bindings.
        
        *Args*:
        
            ``complex``: The entry in self.complexes for the complex
            
            ``amount`` (float): The amount of the complex to be bound
            
        """
        cdef double new_quantity, factor
        cdef Bond bond
        cdef list binding_list = [bond for bond in complex.binding_list 
                                    if bond.other.amount > 0.000001]
        cdef list new_complexes
        cdef CEntry other, result
        cdef np.ndarray[nfloat_t, ndim=1] function = self.function
        cdef int lb = len(binding_list) 
        cdef int i
        if lb > 0: 
            #scales so no more than gv.maximum_bound of the complex binds
            #                    affinty*current amount of other
            factor = sum([bond.affinity*bond.other.amount 
                         for bond in binding_list])
            if factor > 0:
                factor = min(1,1/factor)*gv.maximum_bound*amount
            else:
                print str(binding_list[0].other.amount)
                factor = 1
            new_complexes = []
            for i in range(lb):
                bond = binding_list[i]
                new_quantity = factor*bond.affinity*bond.other.amount
                other = bond.other
                result = bond.result
                #update complex
                complex.amount -= new_quantity
                #update other
                other.amount -= new_quantity 
                #update result
                result.amount += new_quantity
                #update function
                function += (result.complex.function-other.complex.function-\
                    complex.complex.function)*new_quantity
                
                if gv.complex_complex_binding:
                    new_complexes.append((result,new_quantity))
            if gv.complex_complex_binding:
                for c in new_complexes: #bind the results appropriately
                    self.bind_complex(c[0],c[1])
                    
    cdef calculate_binders(self, CEntry complex):

        """Calculate which complexes a new complex binds to.

        Checks all complexes in the organism to see if they bind to a given 
        complex. Updates both entries accordingly.
        
        *Args*:
        
            ``complex``: The entry in self.complexes for the new complex
        """
        for other_complex in self.complexes.values():
            result_fam = self.environment.families.binds(complex.complex.family,
                                                other_complex.complex.family)
            if result_fam:
                result_name  = str(sorted(eval(complex.complex.name) + 
                                   eval(other_complex.complex.name)))
                try:
                    result = self.complexes[result_name]
                    result_complex = result.complex
                except KeyError:
                    result_complex = self.environment.complexes.get_complex(
                                                        result_name, result_fam)
                    result = CEntry(result_complex, 0.0)
                    self.complexes[result_name] = result
                    self.calculate_binders(result)
                complex.binding_list.append(
                        Bond(result_complex.binding_affinity, other_complex, 
                             result))
                other_complex.binding_list.append(
                        Bond(result_complex.binding_affinity, complex, result))

    def _on_movement(self, location, codon_read_number, organism_id):
        """Calls ``_on_movement_callback`` to report movement.

        Called whenever the organism moves. Pass the callback function to
        Organism's constructor with the ``on_movement`` keyword argument to
        set the callback function.
        """
        if self._on_movement_callback is not None:
            self._on_movement_callback(location, codon_read_number,
                                       organism_id)
                
    def read_genome(self):
        """Reads and executes codons from the genome.

        This is the applications inner most loop, and accounts for most of
        the program runtime. If you're looking to optimize, start here.

        Reads one codon of the genome and performs the appropriate actions
            if a mode codon, change mode
            if a chemical codon perform the mode-specific action for that chemical
        """
        # This mess of inline declarations give a vary significant speed boost
        #  in cython.
        cdef int codon, i, mode, pos, slen, cut, maxpl, minpl, mm, p, id
        cdef int codon_read_number
        cdef int rmax, cmax, moves, move_count, row, column, mv, speed
        cdef str pkey
        cdef np.ndarray[nfloat_t, ndim=1] chemicals = self.chemicals
        cdef np.ndarray[nfloat_t, ndim=1] cell = self.cell
        cdef np.ndarray[nfloat_t, ndim=1] function = self.function
        cdef np.ndarray[nint_t, ndim=1] genome = self.genome
        cdef np.ndarray[nfloat_t, ndim = 3] grid = self.environment.grid
        cdef list current_protein = self.current_protein
        cdef list max
        cdef CEntry new_PE
        rmax =  self.environment.dim[0]
        cmax = self.environment.dim[1]
        cdef Complex protein
        cdef int n_gather = gv.num_gather
        cdef int n_move = gv.num_move
        cdef int n_chemicals = gv.num_chemicals
        cdef float g_proportion = gv.gather_proportion
        cdef float f, test
        cdef int prot_cost = (1 if gv.protein_cost else 0)
        pos = self.pos
        slen = self.len
        mode = self.mode
        builds = self.builds
        failed_builds = self.failed_builds
        gathers = self.gathers
        moves = self.moves
        move_count = self.move_count
        speed = gv.move_speed
        row = self.row
        column = self.column
        cut = 0
        maxpl = gv.max_protein_length
        minpl = gv.min_protein_length
        mm = gv.max_moves
        id = self.id
        codon_read_number = self._codon_read_number

        # report initial position
        if codon_read_number == 0:
            self._on_movement((row, column), 0, id)
        
        # originally only one codon was read in this method.
        # grouping all codon reads for an organism in one method gives a 
        # performance boost. So organism 1 does all it's actions, 
        # then organism 2, and so on. This works so long as organisms do not
        # compete with each other.
        for i in xrange(gv.org_step):
            codon = genome[pos]
            if codon < 0:
                if current_protein != []:
                    cut = 1
                move_count = 0
                mode = codon
            else:
                # gather mode
                if mode >= - n_gather and n_gather != 0:
                    # gather the specified chemical - inlined for speed
                    chemicals[codon] += cell[codon]*function[codon]*g_proportion
                    if chemicals[codon] < 0:
                        chemicals[codon] = 0
                    gathers += 1
                elif mode >= -n_gather - n_move:
                    # move towards the specified chemical
                    moves += 1
                    move_count += 1 #number of move commands in a row
                    for step in xrange(speed): #move several times
                        max = [(grid[row,column,codon],0)]
                        #up
                        if row > 0 or gv.wrap:
                            test = grid[(row-1)%rmax,column,codon]
                            if test>max[0][0]:
                                max = [[test,1]]
                            elif test == max[0][0]:
                                max.append([test,1])
                        #right
                        if column < cmax -1 or gv.wrap:
                            test = grid[row,(column+1)%cmax,codon]
                            if test>max[0][0]:
                                max = [[test,2]]
                            elif test == max[0][0]:
                                max.append([test,2])
                        #down
                        if row < rmax -1 or gv.wrap:
                            test = grid[(row+1)%rmax,column,codon]
                            if test>max[0][0]:
                                max = [[test,3]]
                            elif test == max[0][0]:
                                max.append([test,3])
                        #left
                        if column > 0 or gv.wrap:
                            test = grid[row,(column-1)%cmax,codon]
                            if test>max[0][0]:
                                max = [[test,4]]
                            elif test == max[0][0]:
                                max.append([test,4])
                        if len(max) > 1:
                            mv = choice(max)[1]
                        else:
                            mv = max[0][1] 
                        if mv == 1:
                            row = (row - 1)%rmax
                        elif mv == 2:
                            column = (column + 1)%cmax
                        elif mv == 3:
                            row = (row + 1)%rmax
                        elif mv == 4:
                            column = (column - 1)%cmax
                        cell = grid[row,column]
                        
                    if move_count >= mm: # if it has moved too long
                        # switch to gather mode
                        mode = -n_gather
                        move_count = 0
                    # report movement for recording
                    self._on_movement((row, column), i + codon_read_number, id)
                else: # add to protein
                    builds += 1
                    if not prot_cost: # if proteins are free
                        current_protein.append(codon)
                        if len(current_protein) >= maxpl:
                            cut = 1
                     # does the organism have sufficient resources?
                    elif chemicals[codon] >= 1.0:
                        chemicals[codon] -= 1.0
                        current_protein.append(codon)
                        if len(current_protein) >= maxpl:
                            cut = 1
                    else: #protein construction failed
                        for p in current_protein: #give back spent resources
                            chemicals[p] += 1
                        current_protein = []
                        # switch to gather mode
                        mode = -n_gather
                        failed_builds += 1
            #Cut Protein
            # When a protein is finished, switch to gather mode and add
            # it to the organism, binding to other complexes if possible.
            if cut: 
                cut = 0
                mode = -n_gather # gather mode
                if len(current_protein) >= minpl: # is it long enough?
                    pkey = str([current_protein])
                    try:
                        new_PE = self.complexes[pkey]
                        protein = new_PE.complex
                        new_PE.amount += 1.0
                    except KeyError:
                        protein = self.environment.complexes.get_complex(pkey)
                        new_PE = CEntry(protein, 1.0)
                        self.complexes[pkey] = new_PE
                        self.calculate_binders(new_PE)
                    # be careful not to reassign function and break
                    #  the link to self.function
                    function += protein.function
                    self.bind_complex(new_PE, 1.0) # all bindings are started here
                # protein was too short - give back spent resources
                elif prot_cost:
                    for p in current_protein:
                        chemicals[p] += 1
                current_protein = []
            pos += 1
            if pos >= slen:
                pos = 0

        self.pos = pos
        self.mode = mode
        self.builds = builds
        self.failed_builds = failed_builds
        self.gathers = gathers
        self.moves = moves
        self.move_count = move_count
        self.row = row
        self.column = column
        self.current_protein = current_protein
        self.function = function
        self._codon_read_number = codon_read_number + gv.org_step
        # report the final position
        if self._codon_read_number == gv.generation_length:
            self._on_movement((row, column), self._codon_read_number, id)
            self._codon_read_number = 0
            
    def fitness(self, genome_cost = True):
        """
        The fitness function of the organism.

        Each chemical contributes a set amount to the fitness. The chemicals 
        provide diminishing returns.
        
        *Args*:
        
            ``genome_cost``: if set to False, the genome fitness cost will not be
            subtracted from the organism.
        """
        fit = sum(self.chem_fitness(self.chemicals))
        fit = fit*self.fit_scale
        if genome_cost:
            fit =  fit - self.len*gv.genome_fitness_cost
        if fit < 0:
            fit = 0
        return fit
        
    def __str__(self):
        out = "Organism: %d"%self.id # add ID later?
        out += "\nlocation = [%d, %d]"%(self.row,self.column)
        out += "\ngenome = " + str(self.genome)
        out += "\npos = %i" % self.pos
        out += "\nmode = %i" % self.mode
        out += "\nchemicals = ["
        for c in self.chemicals:
            out += "%3.3f, " % c # this truncates slightly when loading. Is this a Problem?
        out = out[:-2] + "]"
        out += "\nfunction vector = %s"%function_string(self.function)
        out += "\ncomplexes:\n"
        for c in self.complexes.values():
            out += "%s, %s, %f\n"%(c.complex.name,function_string(c.complex.function),c.amount)
        return out + "\n"
    
    def breed(self):
        """
        returns a copy of the organism with possible mutations, chemicals are not kept
        location may be kept based on global variables
        """
        if gv.child_location == 'self':
            r = self.row
            c = self.column
        elif gv.child_location == 'random':
            r,c = self.environment.random_location()
        else:
            raise ValueError('Unrecognized child location: %s'%gv.child_location)
        #Note that this doesn't set the ID, the simulation must do that
        return Organism(self.environment, row=r, column=c, 
                        genome=mutate_genome(self.genome), chemicals=None,
                        fit_scale=self.fit_scale, 
                        ancestry=self.ancestry+[self.id],
                        on_movement=self._on_movement_callback)
        
    def swap(self, other):
        """
        Copies a section of other's genome into its own genome
        """
        self.genome = swap_gene(self.genome, other.genome)
    
    def chem_fitness(self, np.ndarray chemicals):
        """
        Computes the unscaled fitness benefit of a vector of chemicals.

        This is used to calculate an organisms fitness.
        """
        fit = np.zeros(gv.num_chemicals, float)
        for i in range(gv.num_chemicals):
            function_type = gv.chemical_fitnesses[i][0]
            if function_type == 'log':
                fit[i] =  np.log2(chemicals[i] + 1) * gv.chemical_fitnesses[i][1]
            elif function_type == 'power' or function_type == 'limitlog':
                pwr = gv.chemical_fitnesses[i][2]
                fit[i] =  (chemicals[i]**pwr-1)*gv.chemical_fitnesses[i][1]/pwr
            elif function_type == 'root':
                fit[i] =  chemicals[i]**(1.0/gv.chemical_fitnesses[i][1])
            elif function_type == 'linear':
                fit[i] =  chemicals[i]*gv.chemical_fitnesses[i][1]
            elif function_type == 'piecewise_linear':
                raise NotImplemented("piecewise_linear not yet supported")
            else:
                raise ValueError("Invalid fitness function type: {}".format(
                                 function_type))
        return fit
    
def create_function_vector():
    """
    Creates a default function vector. Capable of harvesting n0 complexes, but nothing else.
    """
    cdef np.ndarray vec = np.zeros(gv.num_chemicals,dtype = float)
    for i in range(gv.chem_list[0]):
        vec[i] = 1
    return vec
    







# cython: embedsignature=True
# cython: profile=False

"""Pykaryote Genomes 
"""
import numpy as np
from numpy import concatenate, zeros
from numpy.random import randint
from random import random

from pykaryote.utils.globals import settings as gv


def generate_genome(length):
    """Returns a new genome of the specified length.

    The genome contains either random chemical codons or all 0's. There are non
    mode codons.

    *Args*:
        
        ``length`` (int): the length of the new genome.
    """
    if gv.initial_genome_creation == 'random':
        genome = randint(0, gv.chem_list[0], length)
    elif gv.initial_genome_creation == 'uniform':
        genome = zeros(length, dtype = int)
    return genome

def codon():
    """
    Returns a random valid codon
    """
    if random() < gv.mode_switch_probability:
        return randint(gv.min_codon, 0)
    else:
        return randint(0, gv.num_chemicals)

def mutate_genome(genome):
    """
    Creates a new genome from the old one. The new genome has independent chances of:
        - doubling itself
        - copying a segment
        - deleting a segment
        - point mutation at any codon
        
    *Args*:
    
        ``genome`` (list): the original genome
    """
    if random() < gv.double_probability:
        genome = concatenate((genome,genome))
    if random() < gv.copy_probability:
        genome = copy_gene(genome)
    if gv.genome_cap_ratio: #if non-zero, make sure the genome isn't too long
        genome = genome[:int(gv.generation_length*gv.genome_cap_ratio)]
    if random() < gv.delete_probability:
        genome = delete_gene(genome)
    return point_mutate(genome)

def copy_gene(genome):
    """
    Copies a random segment of the genome and inserts it into a random spot
    
    *Args*:
    
        ``genome`` (list): the original genome
    """
    stop = len(genome)
    inspt = randint(0,stop-1) #destination
    start = randint(0,stop-1) 
    length = cut_del_length(stop)
    if start + length < stop:
        cut = genome[start:start+length]
    else:
        cut = concatenate((genome[start:],genome[:(start+length)%stop]))
    genome = concatenate((genome[:inspt], cut, genome[inspt:]))
    return genome
    
def point_mutate(genome):
    """
    Iterates through the genome with a random probability of mutating each codon
    
    *Args*:
    
        ``genome`` (list): the original genome
    """
    out = []
    for g in genome:
        if random() < gv.point_mutate_chance:
            out.append(codon())
        else:
            out.append(g)
    return out

def cut_del_length(length):
    """
    Returns the length of genome to copy or cut
    
    *Args*:
    
        ``length`` (int): The maximum length allowed.
    """
    cd = min(length, gv.max_cut_del) #which constrains? length or the global max?
    return randint(1,cd+1) # 1 to cd inclusive

def delete_gene(genome):
    """
    If the genome is long enough, deletes a random segment of the genome
    
    *Args*:
    
        ``genome`` (list): the original genome
    """
    stop = len(genome)
    max_len = stop-gv.min_genome_length
    if max_len <= 0:
        return genome
    start = randint(0,stop-1) 
    length = cut_del_length(max_len)
    if start+length < stop:
        new_genome = concatenate((genome[:start],genome[start+length:]))
    else:
        new_genome = genome[(start+length)%stop:start]
    return new_genome
    
def swap_gene(gen1, gen2):
    """
    Copies a portion of gen2 into gen1
    """
    stop1 = len(gen1)
    stop2 = len(gen2)
    inspt = randint(0,stop1-1)
    start = randint(0,stop2-1) 
    length = cut_del_length(stop2)
    if start + length < stop2:
        cut = gen2[start:start+length]
    else:
        cut = concatenate((gen2[start:],gen2[:(start+length)%stop2]))
    genome = concatenate((gen1[:inspt], cut, gen1[inspt:]))
    return genome



"""
The analysis module is responsible for tracking and displaying the data from a 
BC simulation. There are two classes, Analyzer which is used to read and 
display stored data, and CreativeAnalyzer which extends Analyzer to allow 
tracking and storing data.
"""
from __future__ import division
from subprocess import call, PIPE

from pylab import *
import matplotlib.pyplot as plt
import matplotlib as mpl
from numpy import array, cumsum, arange, savetxt, concatenate, reshape
import numpy as np
import os
import shutil
import os.path
import time
import warnings

from pykaryote.utils.globals import settings as gv
from pykaryote.utils import environment_draw

color_list = ['blue', 'green', 'red', 'aqua', 'purple', 'yellow',
'black', 'fuchsia', 'gray', 'lime', 'maroon', 'navy', 'olive', 'teal',
'orange', 'silver']

mpl.rcParams['axes.color_cycle'] = color_list

# the spacing from top and bottom for the percentiles
percentile = 0.25

def aggregate_batch(analyzer_dirs, output_analyzer_dir):
    """Aggregate a batch of analyzer directories into a single analyzer
    directory by averaging them.
    """
    ann = Analyzer(analyzer_dirs)
    if not os.path.exists(output_analyzer_dir):
        os.makedirs(output_analyzer_dir)
    for metric_datafile in os.listdir(analyzer_dirs[0]):
        if metric_datafile in ['seed', 'used.cfg', 'num_runs', 'environment.npy']:
            continue
        out_path = os.path.join(output_analyzer_dir, metric_datafile)
        if metric_datafile[:27] == 'time_spent_in_gaussian_gen_':
            # aggregate ``time_spent_in_gaussian_gen`` data
            # data is combined into a 3 dimensional array
            # the first dimension is the run number
            # the second and third dimensions hold the data from that run
            data = ann.quick_read(metric_datafile, mode='split')
            np.save(out_path, data)
        elif metric_datafile[:11] == 'gens_until_':
            mean, stddev, blanks = ann.quick_read(metric_datafile, 
                                                    mode='threshold')
            np.savetxt(out_path, mean)
            np.savetxt('{}.stddev'.format(out_path), stddev)
            np.savetxt('{}.blanks'.format(out_path), blanks)
        else:
            try:
                metric_average = ann.quick_read(metric_datafile)
                np.savetxt(out_path, metric_average)
            except ValueError:
                # file was not a metric datafile, ignore it
                continue
    # record number of runs
    with open(os.path.join(output_analyzer_dir, 'num_runs'), 'w') as f:
        f.write('{}'.format(len(ann.base)))


class Analyzer(object):
    """Reads and plots data collected from a simulation.

    The Analyzer class defines methods for reading and plotting simulation 
    data. Data is read from the files created by a CreativeAnalyzer.
    
    src/analyze.py is a command line usable script to use Analyzer to display 
    stored data.
    
    Note that the various ``plot_...`` methods provided by Analyzer are
    functionally equivalent to a pylab ``plot([some data])`` command. 
    (although they do set their own title)
    
    *Args*: ``data_folders (list)``: A list of strings for the directory(s) 
             containing the analysis data. Can be 1 simulations analyzer/ or 
             multiple ones (but with compatible config file)
        
    *Data*:
        The Analyzer provides access to the analysis data. This access is 
        provided through quick_read(data) and export_as_csv(data). The options 
        for data are:
        
        * ``fitness`` - Average fitness
        
        * ``genome_length`` - Average genome length
        
        * ``chemicals`` - Average amount of each chemical owned
        
        * ``time_spent`` - Breakdown of proportional time spent gathering, 
          moving, building
        
        * ``failed_builds`` - Moles of attempted-protein that were returned 
          because of a lack of resources to finish
        
        * ``complexes`` - Average amount of complexes, broken down by length
        
        * ``complex_diversity`` - Average number of unique complexes, 
          broken down by length
        
        * ``complexes_invented`` - Total number of complexes invented, 
          broken down by length
        
        * ``family_sizes`` - Average family size, broken down by length
        
        * ``family_lengths`` - Number of families, broken down by length
        
        * ``chemical_complexity`` - Complexity score of each chemical. See 
          :py:meth:`compute_chem_complexity()` documentation for details
        
        * ``percent_functional`` - Percent of complexes in use that are 
          functional.
          See :py:meth:`percent_functional()` for more details
        
        * ``component_percent_functional`` - Percent of components of functional
          complexes that are themselves functional.
        
        * ``<#>ancestors`` where # is the number of generations back specified 
          by globals - Number of organisms contributing to the genome # 
          generations ahead
        
        * ``complexity`` - Complexity score of each organism. See 
          :py:meth:`compute_complexity`
        
        * ``irreducible_complexity``- Irreducible complexity score of each 
          organism. See :py:meth:`compute_complexity`
        
        * ``num_distinct_complexes_current``
        
        * ``num_distinct_complexes_current_functional``
        
        * ``num_distinct_complexes_current_nonfunctional``
        
        * ``num_distinct_complexes_ever`` - synonymous with ``complexes_invented``
        
        * ``num_distinct_complexes_ever_functional``
        
        * ``num_distinct_complexes_ever_nonfunctional``
        
        * ``num_families_current``
        
        * ``num_families_current_functional``
        
        * ``num_families_current_nonfunctional``
        
        * ``num_families_ever``
        
        * ``num_families_ever_functional``
        
        * ``num_families_ever_nonfunctional``
        
    """

    def __init__(self, data_folders):
        """Initializes an analyzer.
        Args:
            data_folders: a list of paths to simulation data directories.
        """
        if data_folders == [] or data_folders is None:
            raise Exception("Analyzer must be provided with a list of data "\
                            "folders")
        # if only a string was passed, treat it as a single item list
        if type(data_folders) == str:
            data_folders = [data_folders]

        self.base = data_folders
        try:
            if not gv.is_initialized:
                gv.init_globals(os.path.join(self.base[0], "used.cfg"))
        except:
            raise Exception("Failed to read config: %s" % (
                            os.path.join(self.base[0], "used.cfg")))
        if len(data_folders) == 1:
            self.single = True
        else:
            self.single = False
        # ignore warnings from empty graphs
        warnings.filterwarnings('ignore', r'.*Attempting to set identical '\
                                'bottom==top results.*')
        
    def plot_fitness(self, breakdown = True, **kwargs):
        """
        Plots the average fitness
        
        *Args*:
        
            ``breakdown`` (bool): For single runs, this plot can show regions 
            marking min, lower Q, average, upper Q, max. To disable this feature
            and plot only the average line, set breakdown to False 
            
            ``**kwargs``: a list of keyword args that can be passed to the 
            plotting function
        
        .. image:: pics/fitness.png
            :scale: 50 %
            
        """
        
        if self.single:
            if breakdown:
                min = self.quick_read("fitness_min")
                num = len(min)
                lower = self.quick_read("fitness_lower")[:num]
                avg = self.quick_read("fitness_avg")[:num]
                upper = self.quick_read("fitness_upper")[:num]
                max = self.quick_read("fitness_max")[:num]
                x = arange(0,len(avg))
                fill_between(x, min, max, alpha=0.1)
                fill_between(x, lower, upper, alpha=0.2)
                plot(avg)
            else:
                avg = self.quick_read("fitness_avg")
                plot(avg)
        else:
            min, lower, avg, upper, max = self.quick_read("fitness_avg", 
                                                          mode = "breakdown")
            x = arange(0,len(avg))
            fill_between(x, min, max, alpha=0.1)
            fill_between(x, lower, upper, alpha=0.2)
            plot(avg)
        title("Average Fitness")

    def plot_complexity_and_fitness(self, breakdown=True, 
                                    mass_mode=False, **kwargs):
        """Plots two graphs: one of fitness and one of complexity

        *Args*:
        
            ``breakdown`` (bool): For single runs, this plot can show regions 
            marking min, lower Q, average, upper Q, max. To disable this feature
            and plot only the average line, set breakdown to False 
            
            ``**kwargs``: a list of keyword args that can be passed to the 
            plotting function

        """
        subplot(211)
        self.plot_fitness(breakdown = True, **kwargs)
        subplot(212)
        self.plot_complexity(breakdown = True, mass_mode = False, **kwargs)
        
    def plot_genome_length(self, breakdown = True, **kwargs):
        """
        Plots the average genome length
        
        *Args*:
        
            ``breakdown`` (bool): For single runs, this plot can show regions 
            marking min, lower Q, average, upper Q, max. To disable this feature
            and plot only the average line, set breakdown to False 
            
            ``**kwargs``: a list of keyword args that can be passed to the 
            plotting function
        
        .. image:: pics/genome.png
            :scale: 50 %
        """
        if self.single:
            if breakdown:
                min = self.quick_read("genome_length_min")
                num = len(min)
                lower = self.quick_read("genome_length_lower")[:num]
                avg = self.quick_read("genome_length_avg")[:num]
                upper = self.quick_read("genome_length_upper")[:num]
                max = self.quick_read("genome_length_max")[:num]
                x = arange(0,len(avg))
                fill_between(x, min, max, alpha=0.1)
                fill_between(x, lower, upper, alpha=0.2)
                plot(avg)
            else:
                avg = self.quick_read("genome_length_avg")
                plot(avg)
        else:
            min, lower, avg, upper, max = self.quick_read("genome_length_avg", 
                                                          mode = "breakdown")
            x = arange(0,len(avg))
            fill_between(x, min, max, alpha=0.1)
            fill_between(x, lower, upper, alpha=0.2)
            plot(avg)
        title("Average Genome Length")
        ylim(ymin=0)
        
    def plot_avg_time_spent(self, **kwargs):
        """
        Plots the average time spent on each type of action. The white section
        corresponds to the time spent switching modes in which no useful work
        is done.
        
        *Args*:
            ``**kwargs``: a list of keyword args that can be passed to the 
            plotting function
        
        .. image:: pics/time_spent.png
            :scale: 50 %
        """
        y_data = self.quick_read("time_spent").transpose()/gv.generation_length
        y_data_stacked = cumsum(y_data, axis = 0)
        x = arange(0, len(y_data[0]))
        fill_between(x, 0, y_data_stacked[0,:], 
                     facecolor = (.1,.5,.1), alpha=0.7)
        fill_between(x, y_data_stacked[0,:], y_data_stacked[1,:], 
                     facecolor=(.3,.3,.8), alpha=0.7)
        fill_between(x, y_data_stacked[1,:], y_data_stacked[2,:], 
                     facecolor=(1,.3,.3))
        plot(y_data_stacked[0,:], label = "gather", color = (0,.5,0),**kwargs)
        plot(y_data_stacked[1,:], label = "move", color = (0,0,.8),**kwargs)
        plot(y_data_stacked[2,:], label = "build", color = (1,0,0),**kwargs)
        legend(loc=3, borderaxespad=0.)
        title("Time spent on each action")
        
    def plot_failed_builds(self, **kwargs):
        """
        Plots the average number of moles of attempted-protein that were 
        returned due to insufficient resources to finish.
        
        *Args*:
            ``**kwargs``: a list of keyword args that can be passed to the 
            plotting function
        
        """
        if self.single:
            data = self.quick_read("failed_builds")
            plot(data, **kwargs)
        else:
            min, lower, avg, upper, max = self.quick_read("failed_builds", 
                                                          mode = "breakdown")
            x = arange(0,len(avg))
            fill_between(x, min, max, facecolor = color_list[0], 
                         alpha=0.2, edgecolor = "white")
            fill_between(x, lower, upper, facecolor = color_list[0], 
                         alpha=0.2, edgecolor = "white")
            plot(avg, color = color_list[0])
        xlabel("generations")
        ylabel('moles')
        title("Average moles of failed attempted-protein")
        
    def plot_gen_time(self, **kwargs):
        """
        Plots the time taken by each generation.
        
        *Args*:
            ``**kwargs``: a list of keyword args that can be passed to the 
            plotting function
        
        """
        if self.single:
            data = self.quick_read("gen_time")
            plot(data, **kwargs)
        else:
            min, lower, avg, upper, max = self.quick_read("gen_time", 
                                                          mode = "breakdown")
            x = arange(0,len(avg))
            fill_between(x, min,   max,   facecolor = color_list[0], 
                         alpha=0.2, edgecolor = "white")
            fill_between(x, lower, upper, facecolor = color_list[0], 
                         alpha=0.2, edgecolor = "white")
            plot(avg, color = color_list[0])
        xlabel("generations")
        ylabel('time (seconds)')
        title("Time per generation")
        
    def plot_chemicals(self, **kwargs):
        """
        Plots the average number of each chemical owned by organisms in each 
        generation
        
        *Args*:
            ``**kwargs``: a list of keyword args that can be passed to the 
            plotting function
        
        .. image:: pics/chemicals.png
            :scale: 50 %
        """
        subplots_adjust(right = .8)
        data = self.quick_read("chemicals") + 1
        for c in range(gv.num_chemicals):
            semilogy([gen[c] for gen in data], label=str(c), **kwargs)
        legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0., 
               title = "Chem #")
        title("Average production of chemicals")
        
    def plot_complexes(self, **kwargs):
        """
        Plots the average units of complexes owned by an organism, broken down
        by complex length.
        
        *Args*:
            ``**kwargs``: a list of keyword args that can be passed to the 
            plotting function
        
        .. image:: pics/complexes.png
            :scale: 50 %
        """
        subplots_adjust(right=.8)
        data = self.quick_read("complexes") + 1
        if gv.max_complex_length > 1:
            for c in range(gv.max_complex_length): 
                semilogy(data[: , c], label = str(c+1), **kwargs)
        else:
            semilogy(data,label = "1", **kwargs)
        legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0., 
               title = "Length")
        title("Average units of complexes")
        
    def plot_complex_diversity(self, **kwargs):
        """
        Plots the average number of distinct complexes of each length
        owned by an organism
        
        *Args*:
            ``**kwargs``: a list of keyword args that can be passed to the 
            plotting function
        
        .. image:: pics/complex_diversity.png
            :scale: 50 %
        """
        subplots_adjust(right=.8)
        data = self.quick_read("complex_diversity")
        if gv.max_complex_length > 1:
            for c in range(gv.max_complex_length): 
                plot(data[:,c],label = str(c+1), **kwargs)
        else:
            plot(data,label = "1", **kwargs)
        legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0., 
               title = "Length")
        title("Number of complexes per organism")
    
    def plot_invented_complexes(self, **kwargs):
        """
        Plots the total number of complexes invented
        
        *Args*:
            ``**kwargs``: a list of keyword args that can be passed to the 
            plotting function
        
        .. image:: pics/invented_complexes.png
            :scale: 50 %
        """
        subplots_adjust(right=.8)
        if self.single:
            data = self.quick_read("complexes_invented")
            if gv.max_complex_length > 1:
                for c in range(gv.max_complex_length): 
                    plot(data[:,c],label = str(c+1), **kwargs)
            else:
                plot(data,label = "1", **kwargs)
        else:
            min, lower, avg, upper, max = self.quick_read("complexes_invented", 
                                                          mode = "breakdown")
            x = arange(0,len(avg))
            for c in range(gv.max_complex_length):
                fill_between(x, min[:,c], max[:,c], \
                    facecolor = color_list[c], alpha=0.2, edgecolor = "white")
                fill_between(x, lower[:,c], upper[:,c], 
                    facecolor = color_list[c], alpha=0.2, edgecolor = "white")
                plot(avg[:,c],color = color_list[c],label = str(c+1))
        legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0.,
               title = "Length")
        title("Number of complexes invented")
        ylim(ymin=0)
        
    def plot_percent_functional(self, **kwargs):
        """
        Plots the percent of complexes that are functional
        
        *Args*:
            ``**kwargs``: a list of keyword args that can be passed to the 
            plotting function
        
        .. image:: pics/percent_functional.png
            :scale: 50 %
        """
        subplots_adjust(right=.8)
        data = self.quick_read("percent_functional")
        if gv.max_complex_length > 1:
            for c in range(gv.max_complex_length): 
                plot(data[:,c],label = str(c+1), **kwargs)
        else:
            plot(data,label = "1", **kwargs)
        legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0.)
        title("Percent of complexes that are functional")
        ylim(ymin=0, ymax = 1)
        
    def plot_component_percent_functional(self, **kwargs):
        """
        Plots the percent of components of complexes that are functional
        
        *Args*:
            ``**kwargs``: a list of keyword args that can be passed to the 
            plotting function
        
        .. image:: pics/comp_perc_func.png
            :scale: 50 %
        """
        subplots_adjust(right=.8)
        if self.single:
            data = self.quick_read("component_percent_functional")
            if gv.max_complex_length > 1:
                for c in range(gv.max_complex_length): 
                    plot(data[:,c],label = str(c+1), **kwargs)
            else:
                plot(data,label = "1", **kwargs)
            legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0.,
                   title = "Length")
            title("Percent of complex-components that are functional")
        else:
            data_array = self.quick_read("component_percent_functional",
                                         aggregate = False)
            
        ylim(ymin=0, ymax = 1)
        
    def plot_family_sizes(self, **kwargs):
        """
        Plots the average size of families of each length
        
        *Args*:
            ``**kwargs``: a list of keyword args that can be passed to the 
            plotting function
        
        .. image:: pics/fam_size.png
            :scale: 50 %
        """
        subplots_adjust(right=.8)
        data = self.quick_read("family_sizes") + .1
        if gv.max_complex_length > 1:
            for c in range(gv.max_complex_length): 
                semilogy(data[:,c],label = str(c+1), **kwargs)
        else:
            semilogy(data,label = "1", **kwargs)
        legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0., 
               title = "Length")
        title("Average Family Size")
        
    def plot_family_lengths(self, **kwargs):
        """
        Plots the number of families of each length
        
        *Args*:
            ``**kwargs``: a list of keyword args that can be passed to the 
            plotting function
        
        .. image:: pics/fam_length.png
            :scale: 50 %
        """
        subplots_adjust(right=.8)
        if self.single:
            data = self.quick_read("family_lengths")
            if gv.max_complex_length > 1:
                for c in range(gv.max_complex_length): 
                    plot(data[:,c],label = str(c+1), **kwargs)
            else:
                plot(data,label = "1", **kwargs)
        else:
            min, lower, avg, upper, max = self.quick_read("family_lengths",
                                                          mode = "breakdown")
            x = arange(0,len(avg))
            for c in range(gv.max_complex_length):
                fill_between(x, min[:,c], max[:,c], \
                    facecolor = color_list[c], alpha=0.2, edgecolor = "white")
                fill_between(x, lower[:,c], upper[:,c], 
                    facecolor = color_list[c], alpha=0.2, edgecolor = "white")
                plot(avg[:,c],color = color_list[c],label = str(c+1))
        legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0.,
               title = "Length")
        title("Number of Families of a given length")
        ylim(ymin = 0)
        
    def plot_chemical_complexity(self, **kwargs):
        """
        Plots the average complexity value for each chemical
        
        *Args*:
            ``**kwargs``: a list of keyword args that can be passed to the 
            plotting function
        
        .. image:: pics/chemical_complexity.png
            :scale: 50 %
        """
        subplots_adjust(right = .8)
        data = self.quick_read("chemical_complexity")
        for c in range(gv.num_chemicals):
            plot([gen[c] for gen in data], label=str(c), **kwargs)
        legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0., 
               title = "Chem #")
        title("Average chemical complexity")
        
    def plot_ancestry(self, **kwargs):
        """
        Plots the number of organisms that contribute to the gene pool a few
        generations ahead. How many controlled by ``track_ancestors``.
        
        *Args*:
            ``**kwargs``: a list of keyword args that can be passed to the 
            plotting function
        
        .. image:: pics/ancestry.png
            :scale: 50 %
        """
        subplots_adjust(right=.8)
        for back in gv.tracked_ancestors:
            try:
                data = self.quick_read("%dancestors"%back)
                plot(data, label=str(back), **kwargs)
            except IOError:
                pass
        legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0.)
        title("Number of contributers N generations ahead")
         
    def plot_num_distinct(self, comp_or_fam, timeframe="current", 
                          functional=None, derivative=False, **kwargs):
        """
        Plots the number of complexes/families that currently/have ever
        existed, and may be restricted to non- or functional.
        
        *Args*:
        
            ``comp_or_fam`` (str) either "complexes" or "families" 
            (only first letter necessary).
            
            ``timeframe`` (str) either "current" or "ever"
            
            ``functional`` whether to plot only functional rather than only 
            non-functional.  Both are printed if this is ``None``.
            
            ``derivative`` whether to plot the derivative.
            
            ``**kwargs``: a list of keyword args that can be passed to the 
            plotting function
        """
        subplots_adjust(right=.8)
        comp_or_fam_arg = "complexes" if comp_or_fam[0]=='c' else "families"
        if functional:
            functional_arg = "_functional"
        elif functional == False:
            functional_arg = "_nonfunctional"
        else:
            functional_arg = ""
        file_to_read = "num_%s_%s%s" % (comp_or_fam_arg, timeframe, 
                                        functional_arg)
        if self.single:
            data = self.quick_read(file_to_read)
            if derivative:
                data = np.diff(data,axis=0)
            if gv.max_complex_length > 1:
                for c in range(gv.max_complex_length): 
                    plot(data[:,c],label = str(c+1), **kwargs)
            else:
                plot(data,label = "1", **kwargs)
        else:
            min, lower, avg, upper, max = self.quick_read(file_to_read,
                                                          mode = "breakdown")
            x = arange(0,len(avg))
            for c in range(gv.max_complex_length):
                fill_between(x, min[:,c], max[:,c], \
                    facecolor = color_list[c], alpha=0.2, edgecolor = "white")
                fill_between(x, lower[:,c], upper[:,c], 
                    facecolor = color_list[c], alpha=0.2, edgecolor = "white")
                plot(avg[:,c],color = color_list[c],label = str(c+1))
        legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0.,
               title = "Length")
        title('%sNumber of distinct %s at %s%s'%(
            ('Derivative of ' if derivative else ''), 
            comp_or_fam, 
            timeframe, 
            ('' if functional==None else " that are"+("" if functional else "n't")+" functional" )))
        xlabel("generations")
        ylabel("number of "+comp_or_fam)
        ylim(ymin=0)

        
    def plot_complexity(self, breakdown = True, mass_mode = False, **kwargs):
        """
        Plots the complexity, averaging over all chemicals.
        Does not weight based on chemical amounts or effectiveness.
        
        *Args*:
        
            ``breakdown`` (bool): For single runs, this plot can show regions 
            marking min, lower Q, average, upper Q, max. To disable this feature
            and plot only the average line, set breakdown to False 
            
            ``mass_mode`` (bool) - special options for mass_run's plot. 
            Default: ``False``
            
            ``**kwargs``: a list of keyword args that can be passed to the 
            plotting function
        
        .. image:: pics/complexity.png
            :scale: 50 %
        """
        if mass_mode:
            plot(self.quick_read("complexity_avg"),label = "complexity_avg",
                 **kwargs)
            title("Complexity average")
            ylim(ymin = 0)
            return 
        if self.single:
            if breakdown:
                min = self.quick_read("complexity_min")
                num = len(min)
                lower = self.quick_read("complexity_lower")[:num]
                avg = self.quick_read("complexity_avg")[:num]
                upper = self.quick_read("complexity_upper")[:num]
                max = self.quick_read("complexity_max")[:num]
                x = arange(0,len(avg))
                fill_between(x, min, max, alpha=0.1, color = color_list[0])
                fill_between(x, lower, upper, alpha=0.2, color = color_list[0])
                plot(avg, label = "complexity", color = color_list[0])
                min = self.quick_read("irreducible_complexity_min")[:num]
                lower = self.quick_read("irreducible_complexity_lower")[:num]
                avg = self.quick_read("irreducible_complexity_avg")[:num]
                upper = self.quick_read("irreducible_complexity_upper")[:num]
                max = self.quick_read("irreducible_complexity_max")[:num]
                x = arange(0,len(avg))
                fill_between(x, min, max, alpha=0.1, color = color_list[1])
                fill_between(x, lower, upper, alpha=0.2, color = color_list[1])
                plot(avg, label = "irreducible_complexity", 
                     color = color_list[1])
            else:
                avg = self.quick_read("complexity_avg")
                plot(avg, label = "complexity", color = color_list[0])
                avg = self.quick_read("irreducible_complexity_avg")
                plot(avg, label = "irreducible_complexity",
                     color = color_list[1])
        else:
            min, lower, avg, upper, max = self.quick_read("complexity_avg",
                                                          mode = "breakdown")
            x = arange(0,len(avg))
            fill_between(x, min, max, alpha=0.1, color = color_list[0])
            fill_between(x, lower, upper, alpha=0.2, color = color_list[0])
            plot(avg, color = color_list[0], label = "complexity")
            min, lower, avg, upper, max = self.quick_read(
                            "irreducible_complexity_avg", mode = "breakdown")
            fill_between(x, min, max, alpha=0.1, color = color_list[1])
            fill_between(x, lower, upper, alpha=0.2, color = color_list[1])
            plot(avg, color = color_list[1], label = "irr. complexity")
        legend(loc=2)
        title("Complexity")
        ylim(ymin = 0)

    def plot_time_spent_in_gaussian(self):
        """Plot how long each organism spent on each chemical.

        Creates a series of subplots where each bar corresponds to an
        the amount of time an organsim spent in each chemical gaussian.
        Each subplot in the series represents a single generation.

        ``data`` is a three dimentional array where::
            data[run_number, organism, gaussian_index] = number_of_codon_reads
                (number of codon reads while in that gaussian)
        """
        width = 0.35
        datafiles = []
        for filename in os.listdir(self.base[0]):
            if filename[:27] == 'time_spent_in_gaussian_gen_':
                datafiles.append(filename)
        gaussian_colors = []
        datafiles = sorted(datafiles, key=lambda x: int(x[27:-4]))
        for index, datafile in enumerate(datafiles):
            gen_number = int(datafile[27:-4])
            data = self.quick_read(datafile, mode='split', format='binary')
            # if this was a single run, we will need to add another dimension
            if len(data.shape) == 2:
                data = np.array([data])
            # if this was loaded from an aggregate, we will have an extra empty
            # dimension to remove
            if len(data.shape) == 4 and data.shape[0] == 1:
                data = data[0]
            # data must be three dimensional
            assert(len(data.shape) == 3)
            if index == 0:
                # initialize variables on first read
                num_runs = data.shape[0]
                plt.figure(figsize=(8*2*num_runs, 6*len(datafiles)))
                gaussian_colors = environment_draw.make_chemical_colors(
                                                        data.shape[2] -1,
                                                        format='matplotlib')
                # empty space uses a shade of grey
                gaussian_colors.append('0.9')
                ind = np.arange(data.shape[1])
            for run_num in range(num_runs):
                # plot data for run_num at generation gen_number
                subplot_num = num_runs*index+run_num+1
                plt.subplot(len(datafiles), num_runs, subplot_num)
                bottom = np.zeros(data.shape[1], dtype=float)
                for gaussian_index, gaussian_data in enumerate(data[run_num].T):
                    plt.bar(ind, gaussian_data,
                            color=gaussian_colors[gaussian_index],
                            bottom=bottom, width=width, align='center')
                    bottom += gaussian_data
                plt.ylabel('Time')
                plt.title('Time spent on chemical mound '\
                          '(generation {}, run {})'.format(gen_number, run_num))
                plt.xlabel('Organisms')
                plt.xticks(ind+width/2.0, range(len(ind)))

                legend_boxes = [Rectangle((0, 0), 1, 1, color=c)
                                for c in gaussian_colors]
                legend_labels = ['mound {}'.format(x)
                                 for x in range(len(legend_boxes))]
                legend_labels[-1] = 'no mound'
                plt.legend(legend_boxes, legend_labels, loc='best',
                           fancybox=True)

    def save_environment_drawing(self, out_dir):
        """Draws a picture of the chemical environment.

        This function does not use matplotlib.
        """
        grid = np.load(os.path.join(self.base[0], 'environment.npy'))
        environment_draw.draw_environment(grid, os.path.join(
                                        out_dir, 'environment.png'))

    def save_all(self, out_dir):
        """Plots and saves graphs of simulation data.
        """
        def save(name=None):
            """Saves a matplotlib figure to ``out_dir``.

            If ``name`` is not provided, use a numeric name: ``figure_x``.
            """
            if name is None:
                name = 'figure_{}'.format(save.fig_num)
                save.fig_num += 1
            plt.savefig(os.path.join(out_dir, name))

        save.fig_num = 1

        figure()
        subplot(211)
        self.plot_fitness()
        subplot(212)
        self.plot_genome_length()
        save()
        
        figure()
        self.plot_complexity()
        save()
        
        figure()
        self.plot_avg_time_spent()
        save()
        
        figure()
        self.plot_failed_builds()
        save()
        
        figure()
        self.plot_gen_time()
        save()

        self.plot_time_spent_in_gaussian()
        save('time_spent_in_gaussians')

        self.save_environment_drawing(out_dir)
        
        if gv.max_complex_length>0: #will break if no complexes
            figure()
            subplots_adjust(hspace=.25)
            subplot(211)
            self.plot_invented_complexes()
            subplot(212)
            self.plot_family_lengths()
            save()
            
            figure()
            subplot(211)
            self.plot_num_distinct("complexes",timeframe="current",
                                   functional=True)
            subplot(212)
            self.plot_num_distinct("complexes",timeframe="current",
                                   functional=False)
            save()
            
            figure()
            subplot(211)
            self.plot_num_distinct("complexes",timeframe="ever",functional=True)
            subplot(212)
            self.plot_num_distinct("complexes",timeframe="ever",
                                   functional=False)
            save()
            
            figure()
            subplot(211)
            self.plot_num_distinct("families",timeframe="current",
                                   functional=True)
            subplot(212)
            self.plot_num_distinct("families",timeframe="current",
                                   functional=False)
            save()
            
            figure()
            subplot(211)
            self.plot_num_distinct("families",timeframe="ever",functional=True)
            subplot(212)
            self.plot_num_distinct("families",timeframe="ever",functional=False)
            save()

            figure()
            subplot(211)
            self.plot_num_distinct("complexes",timeframe="ever",derivative=True)
            subplot(212)
            self.plot_num_distinct("families",timeframe="ever",derivative=True)
            save()

        # These graphs only make sense for single
        if self.single:
            figure()
            subplots_adjust(hspace=.25)
            subplot(211)
            self.plot_chemicals()
            subplot(212)
            self.plot_chemical_complexity()
            save()

            figure()
            subplot(111)
            self.plot_ancestry()
            save()

            figure()
            self.plot_time_spent_in_gaussian()
            save()
            
            if gv.max_complex_length>0: #will break if no complexes
                figure()
                subplots_adjust(hspace=.25)
                subplot(211)
                self.plot_complexes()
                subplot(212)
                self.plot_complex_diversity()
                save()
                
                figure()
                subplots_adjust(hspace=.25)
                subplot(211)
                self.plot_percent_functional()
                subplot(212)
                self.plot_component_percent_functional()
                save()
        
    def quick_read(self, data, mode = "standard", ndmin=1, format='text'):
        """Loads data as saved by the CreativeAnalyzer class.

        Reads from a simulation or batch of simulations.

        Average, mean, and standard deveation are calculated using the middle
        80%. Number of blanks uses the whole data set.
        
        *Args*:
        
            ``data`` (str): the data file to read from.
            
            ``mode`` (str): The reading mode. Options are:
                * ``standard`` returns the mean of multiple runs
                * ``threshold`` returns the mean, standard deviation, and 
                    number of simulations which did not reach the threshold.
                    Used to aggregate data specified in the [threshold-metrics]
                    section of a simulation's configuration file.
                * ``split`` returns data from multiple runs separately
                * ``stddev`` retuns tuple (avg, stddev)
                * ``breakdown`` returns a tuple (min, lower_part, avg, 
                    upper_part, max)
            ``format`` (str): File format to expect data in. Either 'text' or
                'binary'
            
        *Returns*:
            
            A numpy array
            *or*
            a tuple of arrays
        
        """
        all_data = []
        max_shape = []
        variable_shape = False
        if data[-4:] == '.npy':
            format = 'binary'
        for num, folder in enumerate(self.base):
            data_location = os.path.join(folder, data)
            if format == 'text':
                arr = np.loadtxt(data_location, dtype=float, ndmin=ndmin)
            elif format == 'binary':
                arr = np.load(data_location)
            all_data.append(arr)
            sim_data = all_data[-1]
            if num == 0:
                max_shape = list(sim_data.shape)
            # check if data has a variable shape
            # find the maximum for each dimension
            for i, size in enumerate(sim_data.shape):
                if len(max_shape) <= i:
                    variable_shape = True
                    max_shape.append(size)
                elif size > max_shape[i]:
                    variable_shape = True
                    max_shape[i] = size
                elif size != max_shape[i]:
                    variable_shape = True
        if variable_shape:
            # resize arrays to maximum size
            for i in range(len(all_data)):
                # 
                a = np.require(all_data[i], requirements='O')
                a.resize(max_shape, refcheck=False)
                all_data[i] = a
        data_array = np.array(all_data)
        if mode == "split":
            # return an array holding the array from each run
            return data_array
        data_array.sort(axis=0)
        if mode == "threshold":
            # calculate number of blanks (-1) in each row
            blanks = np.empty(data_array.shape[1], dtype=int)
            for row_num, row in enumerate(data_array.T):
                blanks[row_num] = np.searchsorted(row, [0.0])[0]
        # calculate middle 80 %
        # Note: this is not actually the middle 80%, it is an approximation,
        # where the upper and lower bounds are rounded.
        # An exact measure of the middle 80% would require taking the weighted
        # average of the bounds instead of rounding.
        # it was decided that rounding is a better solution, as it ensures
        # that actual data is used.
        lower_threshold = (1.0 - gv.mean_to_aggregate) / 2.0
        upper_threshold = gv.mean_to_aggregate + lower_threshold
        lower = int(data_array.shape[0] * lower_threshold)
        upper = int(ceil(data_array.shape[0] * upper_threshold))
        data_array = data_array[lower: upper]
        if mode == "standard":
            return np.average(data_array, axis=0)
        elif mode == "threshold":
            # return the mean, standard deviation, and number of blanks
            # for mean and standard deviation, blanks (-1) are replaced with
            # the max generation number
            if sum(blanks):
                # replace the blanks with the max generation number
                if not gv.generation_limit:
                    replacement = 5000.0
                else:
                    replacement = float(gv.generation_limit)
                data_array[data_array == -1.0] = replacement
            mean = np.mean(data_array, axis=0, dtype=int)
            stddev = np.std(data_array, axis=0, dtype=int)
            return mean, stddev, blanks
        elif mode == "stddev":
            return np.mean(data_array, axis=0), np.std(data_array, axis=0)
        elif mode == "breakdown":
            if self.single:
                raise Exception("Breakdown only valid for Analyzers with \
                                multiple runs")
            return breakdown(data_array)
        else:
            raise ValueError("Unrecognized mode: %s" % mode)
    
    def export_varialble_as_csv(self, data, output_file):
        """
        Converts an analyzer data log to csv.
        
        *Args*:
        
            ``data`` (str): the data file to read from.
            
            ``output_file`` (str): The filename of the new csv data
        
        """
        read_data = array(self.quick_read(data))
        savetxt(output_file, read_data, delimiter=",")
        
    def export_all_as_csv(self, output_file):
        """
        Exports all the analyzer data to a csv with headings. Note that for that 
        is separated by chemical number or complex length columns are created 
        for each chemical or length e.g. chemicals_0, chemicals_1,...
        
        All headings match the data variables discussed in the Analyzer class 
        documentation.
        
        *Args*:
        
            ``output_file`` (str): The filename of the new csv data
        """
        headings = "Generation, fitness_min, "
        fit = self.quick_read("fitness_min")
        data = concatenate( (reshape(arange(0,len(fit)),(-1,1)),
                           reshape(fit,(-1,1))) , axis = 1)
        data, headings = self.add_data(data, headings,"fitness_lower")
        data, headings = self.add_data(data, headings,"fitness_avg")
        data, headings = self.add_data(data, headings,"fitness_upper")
        data, headings = self.add_data(data, headings,"fitness_max")
        data, headings = self.add_data(data, headings,"genome_length_min")
        data, headings = self.add_data(data, headings,"genome_length_lower")
        data, headings = self.add_data(data, headings,"genome_length_avg")
        data, headings = self.add_data(data, headings,"genome_length_upper")
        data, headings = self.add_data(data, headings,"genome_length_max")
        data, headings = self.add_data(data, headings,"chemicals",
                                       gv.num_chemicals)
        #special handling
        data = concatenate((data, self.quick_read("time_spent")), axis = 1)
        headings += "time_gathering, time_moving, time_building, "
        data, headings = self.add_data(data, headings,"chemical_complexity",
                                       gv.num_chemicals)
        data, headings = self.add_data(data, headings,"complexes",
                                       gv.max_complex_length)
        data, headings = self.add_data(data, headings,"complex_diversity",
                                       gv.max_complex_length)
        data, headings = self.add_data(data, headings,"family_sizes",
                                       gv.max_complex_length)
        data, headings = self.add_data(data, headings,"family_lengths",
                                       gv.max_complex_length)
        data, headings = self.add_data(data, headings,"percent_functional",
                                       gv.max_complex_length)
        data, headings = self.add_data(data, headings,
                                       "component_percent_functional",
                                       gv.max_complex_length)
        for g in gv.tracked_ancestors:
            empty_column = np.zeros((len(data),1))
            headings += "ancestors_%d_ahead, "%g
            a_data = self.quick_read("%dancestors"%g)
            for i in range(len(a_data)):
                empty_column[i,0] = a_data[i]
            data = np.concatenate((data, empty_column), axis = 1)
            
        # make sure the directory exists
        try:
            out_path = os.path.split(output_file)[0]
            os.makedirs(out_path)
        except OSError:
            pass
        
        savetxt(output_file, data, delimiter=",")
        # work around to insert headings
        file = open(output_file,'r')
        data_str = file.read()
        file.close()
        file = open(output_file,'w')
        file.write(headings[:-2] + "\n" + data_str)
        file.close()
    
    def add_data(self, data, headings, name, number = 1):
        if number == 1:
            headings += name + ", "
            new_data = reshape(self.quick_read(name),(-1,1))
        else:
            headings += "".join([name+"_%d, "%i for i in range(number)])
            new_data = self.quick_read(name)
        data = concatenate(  (data,new_data ), axis = 1 )  
        return data, headings


class CreativeAnalyzer(object):
    """Saves data from a running simulation.

    Data, in the form of text files containing numpy arrays, are saved to 
    an 'analyzer' directory.
    
    *Args*:
        ``sim``: The simulation for which data will be recorded.
    """
    
    def __init__(self, sim):
        """
        Creates a CreativeAnalyzer for a simulation

        Args:
            sim: The simulation which this comparison is based off of.
            if sim.Comparison is true, this simulation is part of a comparison.
                In that case, don't calculate min, lower, and upper
                for metrics.
        """
        self.sim = sim
        self.base =  os.path.join(self.sim.data, self.sim.name, "analyzer")
        if not gv.is_initialized:
            gv.init_globals(os.path.join(self.base, "used.cfg"))
        gv.prevTime = time.time()
        # initialize lists to record the number of generations it takes for
        # a metric to reach a threshold value
        self._threshold_metrics = {}
        for metric, thresholds in zip(gv.metrics, gv.thresholds):
            l = [-1 for x in range(len(thresholds))]
            self._threshold_metrics[metric] = l
        # dictionary to hold the current value of each metric
        self._current_metric_values = {}
        self.__open_data_files()
        #write seed
        with open(os.path.join(self.base, "seed"), 'w') as f:
            f.write(str(gv.randomseed)+'\n')

    def __open_data_files(self):
        """Get open file handles to all data files.

        Open files are stored in self.data_files, a dictionary mapping
        metric_name `str` to a file like object.
        """
        self.data_files = {}
        metrics = ['fitness_avg', 'genome_length_avg', 'chemicals',
        'time_spent', 'complexes', 'complex_diversity', 'complexes_invented',
        'family_sizes', 'family_lengths', 'chemical_complexity',
        'percent_functional', 'component_percent_functional', 'complexity_avg',
        'irreducible_complexity_avg', 'failed_builds', 'num_complexes_current',
        'num_complexes_current_functional', 
        'num_complexes_current_nonfunctional', 'num_complexes_ever', 
        'num_complexes_ever_functional', 'num_complexes_ever_nonfunctional',
        'num_families_current', 'num_families_current_functional',
        'num_families_current_nonfunctional', 'num_families_ever', 
        'num_families_ever_functional', 'num_families_ever_nonfunctional',
        'gen_time']
        for prev_gen_number in gv.tracked_ancestors:
            metrics.append('%dancestors' % prev_gen_number)
        metrics_not_comparison = ['complexity_min', 'complexity_lower',
        'complexity_upper', 'complexity_max', 'irreducible_complexity_min',
        'irreducible_complexity_lower', 'irreducible_complexity_upper',
        'irreducible_complexity_max', 'fitness_min', 'fitness_lower',
        'fitness_upper', 'fitness_max', 'genome_length_min',
        'genome_length_lower', 'genome_length_upper', 'genome_length_max']
        for metric in metrics:
            filename = os.path.join(self.base, metric)
            self.data_files[metric] = open(filename, 'w')
        # don't track min, lower, upper, max when running a comparison
        if not self.sim.comparison:
            for metric in metrics_not_comparison:
                filename = os.path.join(self.base, metric)
                self.data_files[metric] = open(filename, 'w')

    def __del__(self):
        """Closes metric data files.
        """
        #TODO: use 'with' to open files. Don't explicitly close them.
        self.close_all()

    def save_as_data(self, ndarray, name, format='text'):
        """Saves the given array as a datafile with name ``name``.

        ``format`` (str): either 'text' or 'binary'

        If it is a 1 or 2d array, save in text format.
        If it has three or more dimensions, save in .np binary format.
        """
        if format == 'binary':
            np.save(os.path.join(self.base, name), ndarray)
        elif format == 'text':
            np.savetxt(os.path.join(self.base, name), ndarray)

    def close_all(self):
        """Closes metric data files.

        Also writes threshold metric data files.
        """
        for f in self.data_files.itervalues():
            f.close()
        # write threshold metric data
        for metric, values in self._threshold_metrics.iteritems():
            fname = os.path.join(self.base, 'gens_until_{}'.format(metric))
            with open(fname, 'w') as f:
                for gen_num in values:
                    f.write('{}\n'.format(gen_num))
        # write environment grid
        np.save(os.path.join(self.base, 'environment'),
                self.sim.environment.grid)
        
    def update(self):
        """
        Updates the Analyzer with data from a new generation
        """
        pts = array([org.fitness() for org in self.sim.organisms])
        # if this simulation is a comparison, only record the average
        if self.sim.comparison:
            self.quick_append('fitness_avg', np.average(pts))
        else:
            fi_min, fi_lower, fi_avg, fi_upper, fi_max = breakdown_points(pts)
            self.quick_append("fitness_min", fi_min)
            self.quick_append("fitness_lower", fi_lower)
            self.quick_append("fitness_avg", fi_avg)
            self.quick_append("fitness_upper", fi_upper)
            self.quick_append("fitness_max", fi_max)
        
        pts = array([len(org.genome) for org in self.sim.organisms])
        if self.sim.comparison:
            self.quick_append('genome_length_avg', np.average(pts))
        else:
            gl_min, gl_lower, gl_avg, gl_upper, gl_max = breakdown_points(pts)
            self.quick_append("genome_length_min", gl_min)
            self.quick_append("genome_length_lower", gl_lower)
            self.quick_append("genome_length_avg", gl_avg)
            self.quick_append("genome_length_upper", gl_upper)
            self.quick_append("genome_length_max", gl_max)
        
        chemall = [org.chemicals for org in self.sim.organisms]
        chemicals = [avg([chem[i] for chem in chemall]) 
                                            for i in range(gv.num_chemicals)]
        self.quick_append("chemicals", chemicals)
        
        moves = avg([org.moves for org in self.sim.organisms])
        gathers = avg([org.gathers for org in self.sim.organisms])
        builds = avg([org.builds for org in self.sim.organisms])
        self.quick_append("time_spent", [gathers, moves, builds])
        
        complexes = np.zeros(gv.max_complex_length, float)
        complex_diversity = np.zeros(gv.max_complex_length, float)
        for org in self.sim.organisms:
            for comp in org.complexes.values():
                complexes[comp.complex.length-1] += comp.amount
                complex_diversity[comp.complex.length-1] += 1
        complexes /= gv.num_organisms
        complex_diversity /= gv.num_organisms
        self.quick_append("complexes", list(complexes))
        self.quick_append("complex_diversity", list(complex_diversity))
        
        invented_complexes = [0 for _ in range(gv.max_complex_length)]
        for c in self.sim.environment.complexes.complex_lookup.values():
            invented_complexes[c.length-1] += 1
        self.quick_append("complexes_invented", invented_complexes)
        
        fam_sizes = [0 for _ in range(gv.max_complex_length)]
        fam_lengths = [0 for _ in range(gv.max_complex_length)]
        for fam in self.sim.environment.families.family_lookup.values():
            fam_sizes[fam.length-1] += fam.size
            fam_lengths[fam.length-1] += 1
        fam_sizes = [fam_sizes[i]/fam_lengths[i] if fam_lengths[i] !=0 else 0 for i in range(gv.max_complex_length)]
        self.quick_append("family_sizes", fam_sizes)
        self.quick_append("family_lengths", fam_lengths)
        
        chem_complexities = array([compute_chem_complexity(org) for org in self.sim.organisms])
        chem_comp = [sum(chem_complexities[:,i])/gv.num_organisms for i in range(gv.num_chemicals)]
        self.quick_append("chemical_complexity", chem_comp)
        
        p_func = array([percent_functional(org) for org in self.sim.organisms])
        avg_p_func = [sum(p_func[:,i])/gv.num_organisms for i in range(gv.max_complex_length)]
        self.quick_append("percent_functional", avg_p_func)
        
        c_p_func = array([component_percent_functional(org) for org in self.sim.organisms])
        avg_c_p_func = [sum(c_p_func[:,i])/gv.num_organisms for i in range(gv.max_complex_length)]
        self.quick_append("component_percent_functional", avg_c_p_func)
        
        ci_array = array([compute_complexity(org) for org in self.sim.organisms])
        if self.sim.comparison:
            c_avg = np.average(ci_array[:,0])
            ic_avg = np.average(ci_array[:,1])
            self.quick_append('complexity_avg', c_avg)
            self.quick_append('irreducible_complexity_avg', ic_avg)
        else:
            c_min, c_lower, c_avg, c_upper, c_max = breakdown_points(
                                                                ci_array[:,0])
            self.quick_append("complexity_min", c_min)
            self.quick_append("complexity_lower", c_lower)
            self.quick_append("complexity_avg", c_avg)
            self.quick_append("complexity_upper", c_upper)
            self.quick_append("complexity_max", c_max)
            ic_min, ic_lower, ic_avg, ic_upper, ic_max = breakdown_points(
                                                                ci_array[:,1])
            self.quick_append("irreducible_complexity_min", ic_min)
            self.quick_append("irreducible_complexity_lower", ic_lower)
            self.quick_append("irreducible_complexity_avg", ic_avg)
            self.quick_append("irreducible_complexity_upper", ic_upper)
            self.quick_append("irreducible_complexity_max", ic_max)
        
        #for finish_criteria
        self.sim.current_complexity_avg = c_avg
        self.sim.current_irreducible_complexity_avg = ic_avg
        
        for back in gv.tracked_ancestors:
            ancestor_set = set([]) # no duplicates
            for org in self.sim.organisms:
                #append the id of this org's ancestor `back` generations ago
                try:
                    ancestor_set.add(org.ancestry[-back])
                except IndexError:
                    pass
            self.quick_append("%dancestors"%back, len(ancestor_set))

        #failed_builds
        failed_builds = avg([org.failed_builds for org in self.sim.organisms])
        self.quick_append("failed_builds",failed_builds)

        #num distinct complexes current
        cur_complexes = set()
        for org in self.sim.organisms:
            for comp in org.complexes.values():
                cur_complexes.add((comp.complex))
        complexes = np.zeros(gv.max_complex_length, int)
        complexes_f  = np.zeros(gv.max_complex_length, int) #functional
        complexes_nf = np.zeros(gv.max_complex_length, int) #nonfunctional
        for comp in cur_complexes:
            complexes[comp.length-1]+=1
            if comp.is_functional:
                complexes_f[comp.length-1]+=1
            else:
                complexes_nf[comp.length-1]+=1
        self.quick_append("num_complexes_current",list(complexes))
        self.quick_append("num_complexes_current_functional",list(complexes_f))
        self.quick_append("num_complexes_current_nonfunctional",
                          list(complexes_nf))

        #num distinct complexes ever
        complexes = np.zeros(gv.max_complex_length, int)
        complexes_f  = np.zeros(gv.max_complex_length, int) #functional
        complexes_nf = np.zeros(gv.max_complex_length, int) #nonfunctional
        for comp in self.sim.environment.complexes.complex_lookup.values():
            complexes[comp.length-1]+=1
            if comp.is_functional:
                complexes_f[comp.length-1]+=1
            else:
                complexes_nf[comp.length-1]+=1
        self.quick_append("num_complexes_ever",list(complexes))
        self.quick_append("num_complexes_ever_functional",list(complexes_f))
        self.quick_append("num_complexes_ever_nonfunctional",list(complexes_nf))

        #num families current
        cur_fams = set()
        for org in self.sim.organisms:
            for comp in org.complexes.values():
                cur_fams.add((comp.complex.family))
        fams = np.zeros(gv.max_complex_length, int)
        fams_f = np.zeros(gv.max_complex_length, int) #functional
        fams_nf = np.zeros(gv.max_complex_length, int) #nonfunctional
        for fam in cur_fams:
            fams[fam.length-1]+=1
            if max(fam.function_vector) > gv.weak_factor*fam.length*gv.mutate_factor: #it's functional
                fams_f[fam.length-1]+=1
            else:
                fams_nf[fam.length-1]+=1
        self.quick_append("num_families_current",list(fams))
        self.quick_append("num_families_current_functional",list(fams_f))
        self.quick_append("num_families_current_nonfunctional",list(fams_nf))

        #num families ever
        fams = np.zeros(gv.max_complex_length, int)
        fams_f = np.zeros(gv.max_complex_length, int) #functional
        fams_nf = np.zeros(gv.max_complex_length, int) #nonfunctional
        for fam in self.sim.environment.families.family_lookup.values():
            fams[fam.length-1]+=1
            if (max(fam.function_vector) > 
                gv.weak_factor*fam.length*gv.mutate_factor):
                #it's functional
                fams_f[fam.length-1]+=1
            else:
                fams_nf[fam.length-1]+=1
        self.quick_append("num_families_ever",list(fams))
        self.quick_append("num_families_ever_functional",list(fams_f))
        self.quick_append("num_families_ever_nonfunctional",list(fams_nf))

        #gen_time
        curTime=time.time()
        try:
            self.quick_append("gen_time",curTime-gv.prevTime)
        except AttributeError:
            pass
        gv.prevTime=curTime

        # threshold metrics
        # Record how many generations it takes a metric to reach a threshold
        for metric, thresholds in zip(gv.metrics, gv.thresholds):
            for i, threshold in enumerate(thresholds):
                if (self._threshold_metrics[metric][i] == -1 and 
                    self._current_metric_values.get(metric, -1) >= threshold):
                    self._threshold_metrics[metric][i] = self.sim.generation

    def quick_append(self, metric, new_data):
        """
        Appends to the analyzer data logs.
        
        *Args*:
        
            ``metric`` (str): the name of the metric being recorded
            
            ``new_data``: The new data to add
        """
        self._current_metric_values[metric] = new_data
        if type(new_data) == list or type(new_data) == array: #strip brackets
            for value in new_data:
                self.data_files[metric].write('{} '.format(value))
            self.data_files[metric].write('\n')
        else:
            self.data_files[metric].write('{}\n'.format(new_data))
        
def avg(values):
    """
    Computes the arithmetic mean of a list
    """
    return sum(values, 0.0) / len(values)

def compute_chem_complexity(org):
    """
    Computes the "complexity" of each chemical for an organism. 
    Complexity is based on the contribution of complexes to the
    gathering of a chemical. 
    """
    comp_vec = np.zeros(gv.num_chemicals)
    
    for comp in org.complexes.values():
        comp_vec += comp.complex.function*comp.complex.length*comp.amount
    for i in xrange(gv.num_chemicals):
        #If the organism isn't benefitting, it's 0 complexity
        if comp_vec[i] > 0 and org.function[i] > 1: 
            #otherwise the formula fails for some cases
            comp_vec[i] = comp_vec[i]/org.function[i] 
        else:
            comp_vec[i] = 0
    return comp_vec
    
def percent_functional(org):
    """
    Examines the complexes produced by org and determines what percent of each
    length are functional.  Bases the percent on the number of unique complexes,
    not on the amounts in which they are produced.
    
    """
    func_vec = np.zeros(gv.max_complex_length)
    n_vec = np.zeros(gv.max_complex_length)
    for comp in org.complexes.values():
        i = comp.complex.length
        n_vec[i-1] += 1
        if comp.complex.is_functional:
            func_vec[i-1] += 1
    for i in range(gv.max_complex_length):
        if n_vec[i] == 0:
            n_vec[i] = 1
    return func_vec/n_vec
    
def component_percent_functional(org):
    """
    Examines the complexes produced by org and determines, for functional 
    complexes, what percent of protein components are functional
    """
    component_func_vec = np.zeros(gv.max_complex_length)
    component_n_vec = np.zeros(gv.max_complex_length)
    for comp in org.complexes.values():
        i = comp.complex.length
        if comp.complex.is_functional:
            if i > 1:
                component_n_vec[i-1] += 1
                # check which components are functional
                comp_funcs = [org.environment.complexes.get_complex(str([c])).is_functional for c in eval(comp.complex.name)]
                component_func_vec[i-1] += sum(comp_funcs)/i
                # bump up functional complexes w/ no functional bits so they can be seen
                if component_func_vec[i-1] == 0.0:
                    component_func_vec[i-1] = .005*i
    for i in range(gv.max_complex_length):
        if component_n_vec[i] == 0:
            component_n_vec[i] = 1
    return component_func_vec/component_n_vec
    
def compute_complexity(org):
    """
    Computes a score for the complexity and irreducible complexity of the 
    organism based on the complexity level of each chemical function.
    
    More information on computing complexities can be found in :ref:`complexity`
    """
    
    comp_vec = np.zeros(gv.num_chemicals)
    irr_comp_vec = np.zeros(gv.num_chemicals)
    
    for comp in org.complexes.itervalues():
        comp_vec += comp.complex.function*comp.complex.length*comp.amount
        if comp.complex.is_functional:
            #get the number of components that are not functional
            comp_funcs = [(1-org.environment.complexes.get_complex(str([c])).is_functional) for c in eval(comp.complex.name)]
            irr_comp_vec += comp.complex.function*comp.amount*sum(comp_funcs)
    for i in xrange(gv.num_chemicals):
        #If the organism isn't benefiting, it's 0 complexity
        if comp_vec[i] > 0 and org.function[i] > 1: 
            #otherwise the formula fails for some cases
            comp_vec[i] = comp_vec[i]/org.function[i]
            irr_comp_vec[i] = irr_comp_vec[i]/org.function[i]
        else:
            comp_vec[i] = 0
            irr_comp_vec[i] = 0
            

    complexity = sum(comp_vec)/gv.num_chemicals
    irreducible_complexity = sum(irr_comp_vec)/gv.num_chemicals
    return complexity, irreducible_complexity
    
def breakdown(all_data):
    """
    Breaks down a multi-generation data array and computes the min, lower Q, 
    avg, upper Q, and max for each variable for each generation.
    """
    min = np.amin(all_data,axis = 0)
    avg = np.average(all_data, axis = 0)
    max = np.amax(all_data, axis = 0)
    lower = np.empty_like(avg)
    upper = np.empty_like(avg)
    num = len(all_data)
    fuzzy_index = (num-1)*percentile
    index = int(fuzzy_index)
    offset =  fuzzy_index - index
    all_data = array(all_data)
    #one data point per generation
    if avg.ndim == 1:
        for i in range(len(all_data[0])):
            arr = np.sort(all_data[:,i])
            lower[i] = (1-offset) * arr[index] + (offset)*arr[index+1]
            upper[i] = (1-offset) * arr[-index-1] + (offset)*arr[-index-2]
    #multiple data points per generation
    else:
        for i in range(len(all_data[0])):
            arr = np.sort(all_data[:,i,:],axis = 0)
            lower[i] = (1-offset) * arr[index,:] + (offset)*arr[index+1,:]
            upper[i] = (1-offset) * arr[-index-1,:] + (offset)*arr[-index-2,:]
    return min, lower, avg, upper, max

def breakdown_points(points):
    """
    Computes and returns min, lower Q, avg, upper Q, max of an array of points
    """
    if len(points) == 1:
        return points, points, points, points, points
    min = np.amin(points)
    avg = np.average(points)
    max = np.amax(points)
    fuzzy_index = (len(points)-1)*percentile
    index = int(fuzzy_index)
    offset =  fuzzy_index - index
    arr = np.sort(points)
    lower = (1-offset) * arr[index] + (offset)*arr[index+1]
    upper = (1-offset) * arr[-index-1] + (offset)*arr[-index-2]
    return min, lower, avg, upper, max



#!/usr/bin/env python
"""Code to execute on master thread.

The master thread holds a thread pool and a queue of jobs. It sends jobs to
worker processors and reports status on running jobs.
"""
from mpi4py import MPI
import os.path
import shutil
import time
import datetime
import ConfigParser
from collections import deque
import tags
from pykaryote.utils.comparison import get_config as load_from_cmp
from pykaryote.utils.comparison import make_cfg

comm = MPI.COMM_WORLD


class Master(object):
    """Object to run on the master processor.

    Master controls the queuing of jobs and the reporting of job status.
    """
    def __init__(self, dest_dir, status_interval=10):
        # the directory where mpi jobs are saved
        self.dest_dir = dest_dir
        # the amount of time (in seconds) to wait before printing a status
        # update. default is 10 seconds.
        self._status_interval = status_interval
        # the time when the last status update was displayed
        self._last_status = time.time()

        # data structures to hold the processor pool and queue of jobs
        self._idle_processes = deque(range(1, comm.Get_size()))
        self._busy_processes = set()
        self._waiting_jobs = deque()
        self._running_jobs = []
        self._finished_jobs = []
        self._comparisons = {}
        # flag - true when inside the _wait_and_log() method
        # indicates that Master is waiting for status and it is safe to send
        # send queued jobs
        self._running = False

    def log(self, string):
        """Prints a string to the log.

        The function could later be used to write to a log file.
        """
        print string

    def _processes_needed(self):
        """Returns the number of processes needed to finish the job.

        Used to know how many idle processes to shutdown.
        """
        return len(self._waiting_jobs)

    def add_comparisons(self, batch_config, num_runs=None, clean=True):
        """Adds comparisons from a config file.

        Optionally overrides the number of runs.
        Config files are in .ini format, where each section is the name of a
        comparison. For example:
        
            [my-comparison]
            comparison_config = configs/cmp.cfg
            simulation_config = configs/sim.cfg
            num_runs = 80
        """
        config = ConfigParser.RawConfigParser()
        try:
            with open(batch_config) as f:
                config.readfp(f)
        except IOError:
            print 'Error adding batch: no such file \'{}\''.format(batch_config)
            return
        for comparison_name in config.sections():
            cmp_cfg = os.path.expanduser(config.get(comparison_name,
                                         'comparison_config'))
            sim_cfg = os.path.expanduser(config.get(comparison_name,
                                         'simulation_config'))
            if num_runs is None:
                runs = config.getint(comparison_name, 'num_runs')
            else:
                runs = num_runs
            self.add_comparison(cmp_cfg, sim_cfg, comparison_name, runs,
                                clean=clean)

    def add_comparison(self, comparison_config_filename, sim_config_filename,
                       comparison_name, num_runs, clean=True):
        """Queues a comparison to run.
        """
        # load each comparison option into a list
        vary_attrs, values = load_from_cmp(comparison_config_filename)

        # ensure that the comparison has a valid, unused name
        # a comparisons must contain 'omparison' in its name
        if not 'omparison' in comparison_name:
            comparison_name = 'comparison-{}'.format(comparison_name)
        reserved_names = os.listdir(self.dest_dir)
        reserved_names.extend(self._comparisons.keys())
        if comparison_name in reserved_names:
            num = 0
            while '{}-{}'.format(comparison_name, num) in reserved_names:
                num += 1
            comparison_name = '{}-{}'.format(comparison_name, num)

        # create configuration files from cmp.cfg
        config_filenames = []
        config_dir = os.path.join(self.dest_dir, comparison_name,
                                  'sim_configs')
        if not os.path.exists(config_dir):
            os.makedirs(config_dir)

        gen_limits = []
        for x, vals in enumerate(values):
            out_filename = os.path.join(config_dir, 'sim-{}.cfg'.format(x))
            config_filenames.append(out_filename)
            make_cfg(vary_attrs, vals, sim_config_filename, out_filename)
            config = ConfigParser.RawConfigParser()
            config.read(out_filename)
            gen_limits.append(config.getint('global', 'generation_limit'))
        
        # copy comparison config file to comparison directory
        shutil.copyfile(comparison_config_filename, os.path.join(self.dest_dir, 
                        comparison_name, 'cmp.cfg'))

        # store data about this comparison
        comparison = {}
        comparison['num_runs'] = num_runs
        comparison['values'] = values
        comparison['vary_attrs'] = vary_attrs
        comparison['total_processes'] = len(values) * num_runs
        comparison['num_done'] = 0
        comparison['num_failed'] = 0
        comparison['num_running'] = 0
        comparison['num_waiting'] = len(values) * num_runs
        comparison['graphed'] = False
        comparison['current_gens'] = 0
        comparison['clean'] = clean
        # get total number of generations to be run
        target_gens = 0
        for target_gen in gen_limits:
            target_gens += target_gen * num_runs
        comparison['target_gens'] = target_gens
        comparison['gen_limits'] = gen_limits
        self._comparisons[comparison_name] = comparison

        # create jobs and add to waiting jobs list
        for run_num in range(num_runs):
            for sim_num, config_filename in enumerate(config_filenames):
                target_gen = gen_limits[sim_num]
                job = self._create_comparison_job(config_filename, 
                                                  comparison_name, run_num,
                                                  sim_num, target_gen)
                self._add_job(job)
        self.log('added comparison: {}'.format(comparison_name))

    def _create_comparison_job(self, config_filename, comparison_name, run_num,
                               sim_num, target_gen):
        # target_gen is the generation at which the simulation is set to stop
        job = {}
        job['config_filename'] = config_filename
        job['type'] = 'comparison'
        job['comparison_name'] = comparison_name
        job['run_num'] = run_num
        job['sim_num'] = sim_num
        job['status'] = (0, target_gen, -1)
        return job

    def _add_job(self, job, high_priority=False):
        """Queues a job to run.

        High priority jobs are put in the front of the queue. Regular jobs go
        to the back of the queue.
        """
        if high_priority:
            self._waiting_jobs.appendleft(job)
        else:
            self._waiting_jobs.append(job)
        self._send_jobs()

    def _send_jobs(self):
        """Sends waiting jobs to idle processes.
        """
        while self._waiting_jobs and self._idle_processes and self._running:
            job = self._waiting_jobs.popleft()
            process_id = self._idle_processes.popleft()
            job['process_id'] = process_id

            comm.send(job, process_id, tag=tags.JOB_TAG)

            self._running_jobs.append(job)
            self._busy_processes.add(process_id)
            if job['type'] == 'comparison':
                cmp_name = job['comparison_name']
                self._comparisons[cmp_name]['num_running'] += 1
                self._comparisons[cmp_name]['num_waiting'] -= 1

    def wait_and_log(self):
        """Starts the main loop.

        Call this after adding jobs to the queue. This is the main loop of the
        program, which sends jobs and prints status reports.
        """
        start_time = time.time()
        self._running = True
        self._send_jobs()
        self._report_status()
        while self._busy_processes:
            # shutdown unneeded idle processes
            num_needed = self._processes_needed()
            while num_needed < len(self._idle_processes):
                process_id = self._idle_processes.popleft()
                comm.send(-1, process_id, tag=tags.SHUTDOWN_TAG)

            # wait for a worker to message
            status = MPI.Status()
            message = comm.recv(source=MPI.ANY_SOURCE, tag=MPI.ANY_TAG,
                                status=status)

            process_id = status.source
            if status.tag == tags.SUCCESS_TAG:
                self._on_job_done(message, status.tag)
            elif status.tag == tags.FAILED_TAG:
                self._on_job_done(message, status.tag)
            elif status.tag == tags.STATUS_TAG:
                self._on_recive_status(message[0], message[1], message[2],
                                       process_id)
        print 'all jobs done'
        elapsed = datetime.timedelta(seconds=time.time() - start_time)
        print 'runtime: {} (hh:mm:ss)'.format(elapsed)

    def _on_recive_status(self, current_gen, target_gen, eta, process_id):
        """Processes status updates received from running simulations.
        """
        # TODO: store jobs better for O(1) access time
        for job in self._running_jobs:
            if job['process_id'] == process_id:
                job['status'] = (current_gen, target_gen, eta)
                break
        if time.time() - self._last_status > self._status_interval:
            self._report_status()

    def _report_status(self):
        """Reports status on running jobs.

        called at most every self._status_interval seconds.
        """
        for comp in self._comparisons.itervalues():
            comp['current_gens'] = 0

        for job in self._finished_jobs:
            if job['type'] == 'comparison':
                g = job['status'][1]
                self._comparisons[job['comparison_name']]['current_gens'] += g
        for job in self._running_jobs:
            if job['type'] == 'comparison':
                g = job['status'][0]
                self._comparisons[job['comparison_name']]['current_gens'] += g

        total_gens_done = 0
        total_gens_target = 0
        no_target_gen = False

        # generate status strings for each comparison
        comparisons = []
        for name, comp in self._comparisons.iteritems():
            if comp['target_gens'] == 0:
                percent_done = '??'
                no_target_gen = True
            else:
                total_gens_done += comp['current_gens']
                total_gens_target += comp['target_gens']
                percent_done = comp['current_gens'] / float(comp['target_gens'])
                percent_done = '{:.2%}'.format(percent_done)
            header = '{name}: {percent_done}'.format(name=name,
                                                     percent_done=percent_done)
            summary = '({} running, {} done, {} failed, {} queued)'.format(
                                        comp['num_running'], comp['num_done'], 
                                        comp['num_failed'], comp['num_waiting'])
            comparisons.append('{}\n\t{}'.format(header, summary))
        if not total_gens_target:
            no_target_gen = True
        # generate status string for total
        if no_target_gen:
            total_progress = 'NA'
        else:
            percent = total_gens_done / float(total_gens_target)
            total_progress = '{:.2%}'.format(percent)

        # print status information
        print '\nTotal: {} ({}/{})'.format(total_progress, total_gens_done, 
                                         total_gens_target)
        for comparison_status in comparisons:
            print comparison_status
        # print jobs with top 3 ETAs
        print 'Top 3 slowest jobs:'
        sort_key = lambda j: j.get('status', (0, 0, -1))[2]
        for job in sorted(self._running_jobs, key=sort_key, reverse=True)[:3]:
            eta = job.get('status', (0, 0, -1))[2]
            if eta == -1:
                eta_str = 'N/A'
            else:
                eta_str = '{} min {} sec'.format(eta/60, eta%60)
            print '\t{} eta: {}'.format(self._job_to_str(job), eta_str)
        print

        self._last_status = time.time()

    def _on_job_done(self, job, tag):
        """Called when a job is finished or died.

        Checks to see if the entire comparison is finished and sends waiting
        jobs.
        """
        # mark process as idle
        process_id = job['process_id']
        self._busy_processes.remove(process_id)
        self._idle_processes.append(process_id)
        #TODO: store jobs so that access by process id is O(1)
        index = 0
        for j in self._running_jobs:
            if j['process_id'] == process_id:
                break
            index += 1
        job = self._running_jobs.pop(index)
        self._finished_jobs.append(job)

        if job['type'] == 'comparison':
            cmp_name = job['comparison_name']
            comp = self._comparisons[cmp_name]
            # update statistics
            comp['num_running'] -= 1
            # target_gen = comp['gen_limits'][job['sim_num']]
            # comp['current_gens'] += target_gen
            if tag == tags.SUCCESS_TAG:
                # process finished successfully.
                self.log('job done: {}'.format(self._job_to_str(job)))
                comp['num_done'] += 1
            elif tag == tags.FAILED_TAG:
                # process failed or was killed
                self.log('job failed: {}'.format(self._job_to_str(job)))
                comp['num_failed'] += 1
            if self._comparison_ready_to_graph(cmp_name):
                if comp['num_done'] == 0:
                    # don't graph if all the runs failed
                    self.log('finished comparison {}, but all runs failed'.format(cmp_name))
                else:
                    self._send_graph_comparison_job(cmp_name)
                    self.log('graphing {}'.format(cmp_name))
        elif job['type'] == 'comparison_graph':
            cmp_name = job['comparison_name']
            if tag == tags.SUCCESS_TAG:
                self._comparisons[cmp_name]['graphed'] == True
                self.log('finished comparison: {}'.format(cmp_name))
            else:
                self.log('finished comparison: {}, but graphing failed'.format(cmp_name))

        # send waiting jobs
        self._send_jobs()

    def _job_to_str(self, job):
        """Return a human readable string representation of a job.
        """
        if job['type'] == 'comparison':
            return '{comparison_name} sim {sim_num} run {run_num}'.format(**job)
        elif job['type'] == 'comparison_graph':
            return 'graph {}'.format(job['comparison_name'])

    def _comparison_ready_to_graph(self, cmp_name):
        """Returns True if a comparison of the given name is ready to be graphed.
        """
        comp = self._comparisons[cmp_name]
        return (comp['num_done'] + comp['num_failed'] == 
                comp['total_processes'])

    def _send_graph_comparison_job(self, cmp_name):
        """Queues a worker to aggregate and graph a finished comparison.
        """
        job = {}
        job['type'] = 'comparison_graph'
        job['comparison_name'] = cmp_name
        job['values'] = self._comparisons[cmp_name]['values']
        job['num_runs'] = self._comparisons[cmp_name]['num_runs']
        job['clean'] = self._comparisons[cmp_name]['clean']
        self._add_job(job, high_priority=True)

    def shutdown(self):
        """Shuts down all processes and clear the list of queued jobs.

        If a job is currently running, that job will not shutdown until
        it finishes.

        No jobs can be sent after this call.
        """
        for process_id in range(1, comm.Get_size()):
            comm.send(-1, process_id, tag=tags.SHUTDOWN_TAG)
        self._idle_processes = deque()
        self._busy_processes = set()
        self._waiting_jobs = deque()
        self._running_jobs = []
